package cologne.eck.image_pea;

import java.awt.Point;

/*
 * Peafactory - Production of Password Encryption Archives
 * Copyright (C) 2015  Axel von dem Bruch
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License
 * along with this library.
 */

/**
 * Dialog to get the password and start the image pea.
 */

/*
 * Password dialog to decrypt image.
 * Image is displayed in LockFrameImage.
 *
 * Password can not be changed and content (image) can not be modified.
 *
 * cipher text is stored in external file or inside jar
 *
 */



import java.awt.Window;
import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;

import javax.imageio.ImageIO;
import javax.swing.*;
import javax.swing.filechooser.FileFilter;
import javax.swing.filechooser.FileNameExtensionFilter;

import cologne.eck.all_peas.files.FileComposer;
import cologne.eck.all_peas.files.FileModel;
import cologne.eck.all_peas.files.FileTypePanel;
import cologne.eck.all_peas.control.ContentUtilities;
import cologne.eck.all_peas.control.PeaControl;
import cologne.eck.all_peas.data.PeaProperties;
import cologne.eck.all_peas.gui.CursorManager;
import cologne.eck.all_peas.gui.MultiFileSelectionDialog;
import cologne.eck.all_peas.gui.PeaFileChooser;
import cologne.eck.all_peas.gui.PswDialogView;
import cologne.eck.peafactory.crypto.CipherStuff;
import cologne.eck.tools.EntropyPool;
import cologne.eck.tools.FileTools;
import cologne.eck.tools.ReadResources;
import cologne.eck.tools.UnexpectedValueException;
import cologne.eck.tools.Zeroizer;


public final class ImageControl extends PeaControl {

	private static int imageWidth = 800;
	private static int imageHeight = 600;
	protected static boolean resize = true;

	private static PswDialogView dialogView;

	private static ImageControl pswDialog;
	private static LockFrameImage lockFrame = null;

	private static final String VERSION = "1.1";
	private static final String YEAR_OF_PUBLICATION = "2017";
	private static final String WEBSITE = "/html/image_pea.html";
	private static final String SOURCE_LINK = "http://eck.cologne/peafactory/src/downloads/imagePEA/imagePEA-1.0-src.zip";



	private ImageControl() {

		pswDialog = this;
		setDialog(pswDialog);
		dialogView = PswDialogView.getInstance();
		// start the EntropyPool:
		//EntropyPool.getInstance().updateThread(System.nanoTime());

	}

	protected final static ImageControl getInstance() {

		ImageControl dialog = null;
		if (pswDialog == null) {

			dialog = new ImageControl();

		} else {
			// return null
		}
		return dialog;
	}

	public static void main(String[] args) {

		if(args.length > 0) {
			PeaProperties.setWorkingMode(args[0]);
		}
		PswDialogView.setUI(true);

		initializeVariables();

		// settings:
		PeaProperties.setFileType("image");

		//Attachments.setFileIdentifier(PeaSettings.getFileIdentifier());

		pswDialog = ImageControl.getInstance();
		setDialog(pswDialog);	

//		if (PswDialogView.isInitializing() == true) {
			
//			PeaControl.initializePassword();

/*			NewPasswordDialog.setRandomCollector(true);
			NewPasswordDialog newPswDialog = NewPasswordDialog.getInstance(
					ImageControl.dialogView,
					PeaProperties.getBundle().getString("initialization"),
					null, dialogView.getLocation());
			pswDialog.setInitializedPassword( newPswDialog.getDialogInput() );
			NewPasswordDialog.setRandomCollector(false);

			if (newPswDialog.getDialogInput() == null){
				System.exit(0);
			} else {
				PswDialogView.setMessage("select_image", true);
				PeaControl.getDialog().startExecution(false);
			}  */
//		} else {
		ImageControl.dialogView.setVisible(true);
//		}
	}


	//-------------------HELPER-FUNCTIONS--------------

	@Override
	public void clearSecretValues() {
		// is done in LockFrameImage to handle initialization
		//LockFrameImage.clearImageBytes();
	}

	@Override
	public void startDecryption() {

		//ArrayList <String> fileNames = null;
		String[] fileNames = null;

		if (PswDialogView.isInitializing() == false) {

			// get file names:
//			fileNames = PeaControl.getAvailableFileNames();
			// get files to encrypt:
			fileNames = PeaControl.getEncryptedFileTypePanel().getValidFileNames(true);

			setSaltForEncryptedFiles(fileNames);

	/*		if (fileNames.length == 0) {
				PswDialogView.setMessage(PeaProperties.getBundle().getString("no_file_selected"), true);
				return;
			}
			if (fileNames[0].equals(PeaProperties.getBundle().getString("image_inside_jar_archive"))){
				byte[] cipherBytes = ReadResources.getResourceFromJAR("resources/text.lock");
				handleAttachedSalt(cipherBytes);
			} else {

				// get and set the attached salt from first file:
				int saltLen = KeyDerivation.getSaltSize();
				byte[] end = null;
				try {

					end = Attachments.getEndBytesOfFile(
							fileNames[0],// take from first file
							Attachments.getFileIdentifierSize()
							+ saltLen);

					byte[] attachedSalt = new byte[saltLen];
					System.arraycopy(end,  0,  attachedSalt,  0,  saltLen);
					// reset salt if password failed before:
					KeyDerivation.setSalt(attachedSalt);

				} catch (IOException e) {
					PswDialogView.setMessage(
							"###" + // translate here, not in PswDialogView
							PeaProperties.getBundle().getString("file_inappropriate")
							+ fileNames[0]//chosenFileNames[0]
							+ PeaProperties.getBundle().getString("caused_error")
							+ e.toString()
							+ PeaProperties.getBundle().getString("attached_salt_not_found"), true);
					if (PeaProperties.getWorkingMode().equals("-t")){
						System.err.println(e.toString() + " - " + e.getMessage());
						e.printStackTrace();
					}
					return;
				}
			} */
			// KDF:
			// derive the key from password and salt:
			byte[] keyMaterial = getKeyMaterial();
			if (keyMaterial == null) {
					PswDialogView.setLabelText("Key derivation failed");
					PswDialogView.clearPassword();
					dialogView.pack();
					return;
			}

			// check the key:
			// read cipher text from first file
			int firstFileIndex = 0;
			int len = fileNames.length;
			// get first file in fileNames:
			for (int i = 0; i < len; i++) {
				if(new File(fileNames[i]).isFile()) {
					firstFileIndex = i;
					break;
				}
			}
			byte[] cipherText = null;
			if (fileNames[firstFileIndex].equals(PeaProperties.getBundle().getString("image_inside_jar_archive"))){
				cipherText = ReadResources.getResourceFromJAR("resources/text.lock");
			} else {
				cipherText = ReadResources.readExternFile(fileNames[firstFileIndex]);
			}
			if (cipherText == null) {
				if (PeaProperties.getWorkingMode().equals("-t")) {
					new UnexpectedValueException("cipherText", "byte[]", "is null for " + fileNames[firstFileIndex]).printDescription();
				} 		
			}

			// check password identifier
			String error = PeaControl.checkPassword(cipherText, keyMaterial);

			if (error != null) {
				// password failed: display error
				PswDialogView.setMessage(error, true);
				System.err.println("password failed");
				return;

			}
			// store the key:
			CipherStuff.getInstance().getSessionKeyCrypt().storeKey(keyMaterial);

			// set first file as image to show:
			byte[] cipherBytes =  null;
			if (fileNames[0].equals(PeaProperties.getBundle().getString("image_inside_jar_archive"))){
				cipherBytes = ReadResources.getResourceFromJAR("resources/text.lock");
			} else {
				cipherBytes = ReadResources.readExternFile(fileNames[0]);
			}
	    	//byte[] cipherBytes = ReadResources.readExternFile(fileNames.get(0));
	    	// decrypt in RAM:
	    	byte[] imageBytes = CipherStuff.getInstance().decrypt(cipherBytes, null, true );
	    	LockFrameImage.setFileList(new ArrayList<String>(Arrays.asList(fileNames)));

	    	LockFrameImage.setImageBytes(imageBytes);
    		if (lockFrame != null) {
    			lockFrame = LockFrameImage.getInstance(PswDialogView.getView().getLocation());
    			lockFrame.setMessage(fileNames[0]);
    		}
			//CipherStuff.getInstance().getSessionKeyCrypt().storeKey(keyMaterial);

		} else { // initializing...

		}
		// lockFrame:
		if (lockFrame == null) {
			Point p = new Point (100,100);
			try {
				p = PswDialogView.getView().getLocation();
			} catch (Exception e) {
				System.err.println(e.getMessage() + ", " + e.toString());
			}
			lockFrame = LockFrameImage.getInstance(p);
		}

		PswDialogView.getView().setVisible(false);

		lockFrame.setVisible(true);

		EntropyPool.getInstance().stopCollection();
	}

	// not used here:
	@Override
	public void preComputeInThread() {}
	@Override
	public String[] getSelectedFileNames() {

		return ( (FileTypePanel)PeaProperties.getTypePanel()).getValidFileNames(true);
	}

	public static final void setImageWidth(int newWidth) {
		imageWidth = newWidth;
	}
	public static final void setImageHeight(int newHeight) {
		imageHeight = newHeight;
	}
	public static final void setChangeSize(boolean newResize) {
		resize = newResize;
	}
	public static final int getImageWidth(){
		return imageWidth;
	}
	public static final int getImageHeight(){
		return imageHeight;
	}

	@Override
	public void initializeNewFiles() {

		Window win = null;
        if (lockFrame != null && lockFrame.isVisible() == true) {
        	win = lockFrame;

        } else {
        	win = PswDialogView.getView();
        }

		FileFilter imageFilter = new FileNameExtensionFilter(
			    "Image files", ImageIO.getReaderFileSuffixes());

		// show a chooser which shows only image files and directories
		// and which allow to select folders:
		PeaFileChooser chooser = new PeaFileChooser(win);
		chooser.setFileFilter(imageFilter);
		chooser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
		chooser.customizeFileChooser(
				PeaProperties.getBundle().getString("select_images_to_encrypt"),
				PeaProperties.getBundle().getString("select_single_image_or_folder"),
				false);// no hidden files option

		ArrayList<String> list = new ArrayList <String>();

	    int returnVal = chooser.showOpenDialog(win);

	    if(returnVal == JFileChooser.APPROVE_OPTION) {	// ok

	    	boolean containsValidImageFile = false;

	    	try {
	    		// set wait cursor for image checking...
	    		win.setCursor(CursorManager.getWaitCursor());

		    	File selectedFile = chooser.getSelectedFile();
		    	// check if at least one image is in the folder,
		    	// check if images are valid and add valid files to list

		    	File[] containingFiles = null;
		    	if (selectedFile.isFile()){
		    		containingFiles = new File[1];
		    		containingFiles[0] = selectedFile;
		    	} else { // directory
		    		// show warning message for large size or number of files:
		    		FileModel model = ((FileTypePanel)PeaProperties.getTypePanel()).getFileComposer().getFileModel();
		    		boolean printErrors = false; 
		    		if (PeaProperties.getWorkingMode().equals("-t")){
		    			printErrors = false;
		    		}
		    		if (model.checkNumberAndSizeOfFolder(selectedFile, this, printErrors) == false){
		    			return;
		    		}
		    		containingFiles = FileTools.getAllFilesOfFolder(selectedFile, false);


		    		// reset wait cursor
		    		win.setCursor(CursorManager.getWaitCursor());
		    		// get all file including from sub-directories, but no directories
			    	containingFiles = FileTools.getAllFilesOfFolder(selectedFile, true);//selectedFile.listFiles();
			    	//PswDialogView.setMessage("Program is working - this may take some time...");
		    	}

		    	int len = containingFiles.length;
		    	if (len > 3) {
			    	PswDialogView.setMessage(PeaProperties.getBundle().getString("work_in_progress"), false);
		    	}
		    	if (len == 1) {
		    		// for single file, check only if image, not MIME type, not extension:
		    		if (LockFrameImage.checkImage(containingFiles[0], false, false) == true) {
		    			// ad the file name to the list
		    			list.add(containingFiles[0].getAbsolutePath());
		    			containsValidImageFile = true;
		    		}
		    	} else {

			    	for (int i = 0; i < len; i++) {
			    		// check also extension, but not MIME type
			    		if (LockFrameImage.checkImage(containingFiles[i], true, false) == true) {
			    			// ad the file name to the list
			    			list.add(containingFiles[i].getAbsolutePath());
			    			containsValidImageFile = true;
			    		}
			    	}
		    	}
	    	} finally {
				// reset the cursor: checking image files
	    		win.setCursor(CursorManager.getDefaultCursor());
	    	}
	    	if (containsValidImageFile == false) {
	    		JOptionPane.showMessageDialog(
	    				win,
	    				PeaProperties.getBundle().getString("no_image_found"),
	    				PeaProperties.getBundle().getString("error"),
	    				JOptionPane.ERROR_MESSAGE);
				// reset the cursor:
				//win.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
	    		return;
	    	} else {
	    		
	    		setSaltForNewFiles();

		    	// set the session key if not already set:
		/*    	if (KeyDerivation.getSalt() == null) { // not set

		    		if (PeaControl.getAvailableFileNames().size() > 0) {
		    			// image inside jar file:
		    			if (PeaControl.getAvailableFileNames().get(0).equals(PeaProperties.getBundle().getString("image_inside_jar_archive"))){
		    				byte[] cipherBytes = ReadResources.getResourceFromJAR("resources/text.lock");
		    				handleAttachedSalt(cipherBytes);
		    			} else {
			    			// get and set the attached salt from first file:
			    			int saltLen = KeyDerivation.getSaltSize();
			    			byte[] end = null;
			    			try {
			    				end = Attachments.getEndBytesOfFile(
			    						PeaControl.getAvailableFileNames().get(0),//selectedFile.getPath(),//fileNames.get(0),// take from first file
			    						Attachments.getFileIdentifierSize()
			    						+ saltLen);

			    				byte[] attachedSalt = new byte[saltLen];
			    				System.arraycopy(end,  0,  attachedSalt,  0,  saltLen);
			    				// reset salt if password failed before:
			    				KeyDerivation.setSalt(attachedSalt);
			    			} catch (IOException e) {

			    				System.err.println("ImageControl, initializeNewFiles: "
			    						+ "Getting salt from first file (" + PeaControl.getAvailableFileNames().get(0) + ")failed - "
			    						+ "attached salt not found");
			    				if (PeaProperties.getWorkingMode().equals("-t")){
			    					System.err.println(e.toString() + " - " + e.getMessage());
			    					e.printStackTrace();
			    				}
			    				// create new nonce and set
			    				byte[] nonce = new RandomStuff().createRandomBytes(Attachments.getNonceSize());
			    				Attachments.setNonce(nonce);

			    				// create a new salt:
			    				KeyDerivation.setSalt(
			    						new RandomStuff().createRandomBytes(KeyDerivation.getSaltSize()));
			    			}
		    			}
		    		} else { // no salt set before
		    			// create new nonce and set
		    			byte[] nonce = new RandomStuff().createRandomBytes(Attachments.getNonceSize());
		    			Attachments.setNonce(nonce);
	    				// create a new salt:
	    				KeyDerivation.setSalt(
	    						new RandomStuff().createRandomBytes(KeyDerivation.getSaltSize()));
		    		}
		    	}*/
	    		// sort list:
	    		Collections.sort(list);


	    		// show all image files as check boxes and ask to encrypt
	    		MultiFileSelectionDialog ifd = new MultiFileSelectionDialog(win,
	    				win.getLocationOnScreen(),
	    				PeaProperties.getBundle().getString("choose_file_to_encrypt"),
	    				list);
	    		// get selected image files:
	    		list = MultiFileSelectionDialog.getSelectedFiles();
	    		// sort the list
	    		// sorted in MultiFileSelectionDialog Collections.sort(list);
	    		ifd.setVisible(false);
	    		ifd.dispose();

		    	try {
		    		// set wait cursor for encryption
		    		win.setCursor(CursorManager.getWaitCursor());
		    		// derive the key
		    		getKeyMaterial();
					// set the wait cursor again: getKeyMaterial may reset cursor
		    		win.setCursor(CursorManager.getWaitCursor());

					// store session key:
					CipherStuff.getInstance().getSessionKeyCrypt().storeKey(getKeyMaterial());

		    		// encrypt selected image files:
					byte[] keyMat = getKeyMaterial();
					int keyLen = keyMat.length;

		    		for (int i = 0; i < list.size(); i++) {
		    			System.out.println("encrypt: " + list.get(i));
		    			// the key will be overwritten
		    			byte[] key = Arrays.copyOf(keyMat, keyLen);
		    			boolean success = ContentUtilities.saveContent(key,
		    					ReadResources.readExternFile(list.get(i)),
		    					list.get(i));
		    			if (success == false) {
		    				// set the wait cursor again: modal error dialog may reset cursor
				    		win.setCursor(CursorManager.getWaitCursor());
		    			}
		    		}
		    		Zeroizer.zero(keyMat);

		    		// add to available files:
			    	// forget previously shown file names...
			    	PeaControl.setAvailableFileNames(list);
			    	LockFrameImage.setFileList(list);

			    	// set first image:
			    	byte[] cipherBytes = ReadResources.readExternFile(list.get(0));
			    	// decrypt in RAM:
			    	byte[] plainBytes = CipherStuff.getInstance().decrypt(cipherBytes, getKeyMaterial(), true );

		    		LockFrameImage.setImageBytes(plainBytes);
		    		if (lockFrame != null) {
		    			lockFrame.setMessage(list.get(0));
		    		}
		    	} finally {
					// reset the cursor:
		    		win.setCursor(CursorManager.getDefaultCursor());
		    	}
		    }
	    }  
	}

	@Override
	public void openEncryptedFiles() {
		
		FileComposer encryptedComposer = PeaControl.getEncryptedFileTypePanel().getFileComposer();
		FileModel encryptedModel = encryptedComposer.getFileModel();
		boolean added = encryptedComposer.addAction(PeaControl.getEncryptedFileTypePanel(),
				PeaProperties.getBundle().getString("select_files_to_encrypt"), 
				true, false);// hidden files, do not check listed files
		if (added == false) { // no file was added
			System.err.println("no file was added");
			return;
		}
		
		boolean validFileFound = encryptedModel.checkForValidFile();
		if (validFileFound == false) {
			//PswDialogView.setMessage(PeaProperties.getBundle().getString("no_valid_file_found"), true);
			PswDialogView.setMessage(PeaProperties.getBundle().getString("open_or_initialize"), true);
		} else {
			// clear errorMessageLabel
			PswDialogView.setMessage("   ", true);
		}		
		
		/*
		
		
		
		PeaFileChooser chooser = new PeaFileChooser(PswDialogView.getView());

		chooser.customizeFileChooser(
	    		PeaProperties.getBundle().getString("open_file"),
	    		PeaProperties.getBundle().getString("select_single_image_or_folder"),
	    		false);// no hidden files option

		FileFilter imageFilter = new FileNameExtensionFilter(
			    "Image files", ImageIO.getReaderFileSuffixes());

		chooser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
		chooser.setFileFilter( imageFilter );

	    int returnVal = chooser.showOpenDialog(PswDialogView.getView());

	    if(returnVal == JFileChooser.APPROVE_OPTION) {

	    	File file = chooser.getSelectedFile();
	    	String selectedFileName = file.getAbsolutePath();

	    	if (file.isDirectory()) {
	    		// get all included files
	    		String[] containingFileNames = FileTools.getAllFileNamesOfFolder(selectedFileName, true);
	    		int len = containingFileNames.length;
	    		// list for valid files:
	    		ArrayList <String> selectionList = new ArrayList <String>();
	    		for (int i =0; i < len; i++) {
	    			String checkResult = PeaControl.checkFile(containingFileNames[i], false);
			    	if ( checkResult != null ) {
			    		System.err.println("selected file failed: " + containingFileNames[i]);
			    	} else { // no error message, file valid
			    		selectionList.add(containingFileNames[i]);
			    	}
	    		}
	    		if (selectionList.size() > 0) {
	    			// sort list
	    			Collections.sort(selectionList);
	    			// add to PswDialogView
	    			int size = selectionList.size();
	    			for (int i =0;i < size; i++) {
	    				PswDialogView.addAvailableFile(selectionList.get(i), true);
	    			}
	    		}
	    	} else if (file.isFile()) {
		    	String checkResult = PeaControl.checkFile(selectedFileName, false);
		    	if ( checkResult != null ) {

		    		System.err.println("selected file failed: " + selectedFileName);
		    		PswDialogView.setMessage(checkResult, true);
		    		return;

		    	} else { // no error message, file valid

		    		PswDialogView.addAvailableFile(selectedFileName, true);
		    	}
	    	}
	    }*/
	}
	@Override
	public final LockFrameImage getLockFrame() {
		return lockFrame;
	}
	@Override
	public String getVersion() {
		return VERSION;
	}
	@Override
	public String getYearOfPublication() {
		return YEAR_OF_PUBLICATION;
	}
	@Override
	public String getWebsite() {
		return WEBSITE;
	}
	@Override
	public String getSourceLink() {
		return SOURCE_LINK;
	}

	@Override
	public boolean checkFormat(File file, boolean easyCheck, boolean middleCheck, boolean expensiveCheck) {
		// TODO Auto-generated method stub
		//return false;
		boolean checkExtension = false;
		boolean checkMimeType = false;
		if (easyCheck == true) {
			checkExtension = true;
		}
		if (middleCheck == true) {
			checkMimeType= true;
		}
		
		return LockFrameImage.checkImage(file, checkExtension, checkMimeType);
	}

	@Override
	protected byte[] getPlainBytes() {
		// not used 
		return null;
	}
}