package cologne.eck.note_pea;

/*
 * Peafactory - Production of Password Encryption Archives
 * Copyright (C) 2015  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */

/**
 * Main frame of editor pea.  
 */

import java.awt.*;
import java.awt.event.*;
import java.io.File;
import java.util.ArrayList;

import javax.accessibility.AccessibleContext;
import javax.swing.*;
import javax.swing.border.LineBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.text.*;
import javax.swing.text.rtf.RTFEditorKit;
import javax.swing.undo.*;

import cologne.eck.all_peas.control.ContentUtilities;
import cologne.eck.all_peas.control.PathFileManager;
import cologne.eck.all_peas.control.PeaControl;
import cologne.eck.all_peas.data.PeaProperties;
import cologne.eck.all_peas.gui.CursorManager;
import cologne.eck.all_peas.gui.PswDialogView;
import cologne.eck.peafactory.crypto.CipherStuff;
import cologne.eck.tools.Converter;


@SuppressWarnings("serial")
public final class LockFrameEditor extends JFrame 
			implements 
			WindowListener, // windowClosingCommands
			DocumentListener, // to register if changes in text are saved
			ChangeListener { // for JTabbedPane focus change
	
	// this frame
	private static LockFrameEditor frame;

	/**
	 * The tab pane containing one tab for each encrypted
	 * text file. Only one file at the same time is decrypted
	 * and displayed: the file content associated with the 
	 * selected and focused tab. 
	 */
	private static JTabbedPane tabPane;
	
	/** this is the actual focused tab with EditorKit, MouseListener, UndoManager
	 * before the focus is changing
	 */
	private TabComponent lastFocusTab;

	/**
	 * UndoManager, which is always registered to the currently
	 * selected and focused tab (its JTextPane)
	 */
	protected static UndoManager manager;// = new UndoManager();
	protected static RTFEditorKit kit = null;
	
	// if the content of the current document was modified: true
	private static boolean docChangeUnsaved = false;
	
	/**
	 * The associated password dialog, which calls the instance
	 * of this class
	 */
	protected static EditorControl dialog;


	private LockFrameEditor(EditorControl _dialog, Point location) {
				
		frame = this;
		dialog = _dialog;
		
	    this.setIconImage( PswDialogView.getImage() );
	    this.setTitle("NotebookPEA");
	    
		this.addWindowListener(this);// for windowClosing
		
		JPanel contentPane = new JPanel();
		contentPane.setLayout(new BoxLayout(contentPane, BoxLayout.PAGE_AXIS));
		this.setContentPane(contentPane);
		
		JPanel menuPanel = new JPanel();
		menuPanel.setLayout(new BoxLayout(menuPanel, BoxLayout.X_AXIS));
		LockFrameEditorMenu menu = new LockFrameEditorMenu();		
		menuPanel.add(menu);
		menuPanel.setMinimumSize(new Dimension (300, 40));
		menuPanel.add(Box.createHorizontalGlue());
		contentPane.add(menuPanel);
		contentPane.add(Box.createVerticalStrut(10));
		
		JPanel buttonMenuPanel = new JPanel();
		buttonMenuPanel.setLayout(new BoxLayout(buttonMenuPanel, BoxLayout.X_AXIS));
		buttonMenuPanel.setMinimumSize(new Dimension(800, 30));	
		buttonMenuPanel.setMaximumSize(new Dimension(800, 30));		
				
		buttonMenuPanel.add(Box.createHorizontalGlue());		
			// does not work: JTextPane does not save alignment
		JButton leftButton = new JButton();
		leftButton.setAction(new StyledEditorKit.AlignmentAction("=  ", 0));
		leftButton.setMargin(new Insets(0,0,0,15));//top-left-bottom-right
//		leftButton.setBackground(Color.WHITE);
		leftButton.setAlignmentX(LEFT_ALIGNMENT);
		leftButton.setToolTipText(PeaProperties.getBundle().getString("left"));
		leftButton.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		//leftButton.setMaximumSize(new Dimension(25, 20));
		buttonMenuPanel.add(leftButton);
		
		JButton centerButton = new JButton();
		centerButton.setAction(new StyledEditorKit.AlignmentAction(" = ", 1));
		centerButton.setMargin(new Insets(0,7,0,7));//top-left-bottom-right
//		centerButton.setBackground(Color.WHITE);
		centerButton.setAlignmentX(LEFT_ALIGNMENT);
		centerButton.setToolTipText(PeaProperties.getBundle().getString("center"));
		centerButton.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		buttonMenuPanel.add(centerButton);		
		
		JButton rightButton = new JButton();
		rightButton.setAction(new StyledEditorKit.AlignmentAction("  =", 2));
		rightButton.setMargin(new Insets(0,15,0,0));//top-left-bottom-right
//		rightButton.setBackground(Color.WHITE);
		rightButton.setAlignmentX(LEFT_ALIGNMENT);
		rightButton.setToolTipText(PeaProperties.getBundle().getString("right"));
		rightButton.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		buttonMenuPanel.add(rightButton);
		
		buttonMenuPanel.add(Box.createHorizontalStrut(5));
		
		
		JButton colorButton0 = new JButton();
		colorButton0.setAction(new StyledEditorKit.ForegroundAction("",new Color(139,139, 0)));
		colorButton0.setBackground(new Color(139,139, 0));//yellow4
		colorButton0.setPreferredSize(new Dimension(15,15));
		colorButton0.setMaximumSize(new Dimension(15,15));
		colorButton0.setOpaque(true);// more L&F-secure
		colorButton0.setBorder(new LineBorder(new Color(139,139, 0), 8));// more L&F-secure
		buttonMenuPanel.add(colorButton0);

		JButton colorButton1 = new JButton();
		colorButton1.setAction(new StyledEditorKit.ForegroundAction("",new Color(205,102,0)));//Color.ORANGE));
		colorButton1.setBackground(new Color(205,102,0));//darkorange2
		colorButton1.setPreferredSize(new Dimension(15,15));
		colorButton1.setMaximumSize(new Dimension(15,15));
		colorButton1.setOpaque(true);
		colorButton1.setBorder(new LineBorder(new Color(205,102,0), 8));
		buttonMenuPanel.add(colorButton1);
		
		JButton colorButton2 = new JButton();
		colorButton2.setAction(new StyledEditorKit.ForegroundAction("",new Color(205,0,0)));
		colorButton2.setBackground(new Color(205,0,0));//red3
		colorButton2.setPreferredSize(new Dimension(15,15));
		colorButton2.setMaximumSize(new Dimension(15,15));
		colorButton2.setOpaque(true);
		colorButton2.setBorder(new LineBorder(new Color(205,0,0), 8));
		buttonMenuPanel.add(colorButton2);

		JButton colorButton3 = new JButton();
		colorButton3.setAction(new StyledEditorKit.ForegroundAction("",new Color(205,0,205)));
		colorButton3.setBackground(new Color(205,0,205));//magenta3
		colorButton3.setPreferredSize(new Dimension(15,15));
		colorButton3.setMaximumSize(new Dimension(15,15));
		colorButton3.setOpaque(true);
		colorButton3.setBorder(new LineBorder(new Color(205,0,205), 8));
		buttonMenuPanel.add(colorButton3);
		
		JButton colorButton4 = new JButton();
		colorButton4.setAction(new StyledEditorKit.ForegroundAction("",Color.BLUE));
		colorButton4.setBackground(Color.BLUE);
		colorButton4.setPreferredSize(new Dimension(15,15));
		colorButton4.setMaximumSize(new Dimension(15,15));
		colorButton4.setOpaque(true);
		colorButton4.setBorder(new LineBorder(Color.BLUE, 8));
		buttonMenuPanel.add(colorButton4);
		
		JButton colorButton5 = new JButton();
		colorButton5.setAction(new StyledEditorKit.ForegroundAction("",new Color(0,139,139)));
		colorButton5.setBackground(new Color(0,139,139));//cyan4
		colorButton5.setPreferredSize(new Dimension(15,15));
		colorButton5.setMaximumSize(new Dimension(15,15));
		colorButton5.setOpaque(true);
		colorButton5.setBorder(new LineBorder(new Color(0,139,139), 8));
		buttonMenuPanel.add(colorButton5);
		
		JButton colorButton6 = new JButton();
		colorButton6.setAction(new StyledEditorKit.ForegroundAction("",new Color(0,139,0)));
		colorButton6.setBackground(new Color(0,139,0));// green4
		colorButton6.setPreferredSize(new Dimension(15,15));
		colorButton6.setMaximumSize(new Dimension(15,15));
		colorButton6.setOpaque(true);
		colorButton6.setBorder(new LineBorder(new Color(0,139,0), 8));
		buttonMenuPanel.add(colorButton6);		
		
		JButton colorButton8 = new JButton();
		colorButton8.setAction(new StyledEditorKit.ForegroundAction("",Color.DARK_GRAY));
		colorButton8.setBackground(Color.DARK_GRAY);
		colorButton8.setPreferredSize(new Dimension(15,15));
		colorButton8.setMaximumSize(new Dimension(15,15));
		colorButton8.setOpaque(true);
		colorButton8.setBorder(new LineBorder(Color.DARK_GRAY, 8));
		buttonMenuPanel.add(colorButton8);		
		
		JButton colorButton9 = new JButton();
		colorButton9.setAction(new StyledEditorKit.ForegroundAction("",Color.BLACK));
		colorButton9.setBackground(Color.BLACK);
		colorButton9.setPreferredSize(new Dimension(15,15));
		colorButton9.setMaximumSize(new Dimension(15,15));
		colorButton9.setOpaque(true);
		colorButton9.setBorder(new LineBorder(Color.BLACK, 8));
		buttonMenuPanel.add(colorButton9);
		
		buttonMenuPanel.add(Box.createHorizontalStrut(5));

		
		JButton nineButton = new JButton();
		nineButton.setAction(new StyledEditorKit.FontSizeAction("" + 9, 9));
		nineButton.setMargin(new Insets(4,3,4,3));//top-left-bottom-right
		nineButton.setAlignmentX(LEFT_ALIGNMENT);
		nineButton.setToolTipText(PeaProperties.getBundle().getString("font_size") + " " + 9);
		nineButton.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 9));
		buttonMenuPanel.add(nineButton);
		
		JButton twelveButton = new JButton();
		twelveButton.setAction(new StyledEditorKit.FontSizeAction("" + 12, 12));
		twelveButton.setMargin(new Insets(2,0,3,0));//top-left-bottom-right
		twelveButton.setAlignmentX(LEFT_ALIGNMENT);
		twelveButton.setToolTipText(PeaProperties.getBundle().getString("font_size") + " " + 12);
		twelveButton.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		buttonMenuPanel.add(twelveButton);		
		
		JButton fiveteenButton = new JButton();
		fiveteenButton.setAction(new StyledEditorKit.FontSizeAction("" + 15, 15));
		fiveteenButton.setMargin(new Insets(1,0,1,0));//top-left-bottom-right
		fiveteenButton.setAlignmentX(LEFT_ALIGNMENT);
		fiveteenButton.setToolTipText(PeaProperties.getBundle().getString("font_size") + " " + 14);
		fiveteenButton.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 15));
		buttonMenuPanel.add(fiveteenButton);
		
		buttonMenuPanel.add(Box.createHorizontalStrut(5));

		
		JButton boldButton = new JButton();
		boldButton.setAction(new StyledEditorKit.BoldAction());
		boldButton.setText(PeaProperties.getBundle().getString("bold_mnemonic"));
		boldButton.setToolTipText(PeaProperties.getBundle().getString("bold"));
		boldButton.setFont(new Font(Font.SANS_SERIF, Font.BOLD, 12));
		boldButton.setMargin(new Insets(2,5,2,5));//top-left-bottom-right
		buttonMenuPanel.add(boldButton);

		JButton italicButton = new JButton();
		italicButton.setAction(new StyledEditorKit.ItalicAction());
		italicButton.setText(PeaProperties.getBundle().getString("italic_mnemonic"));
		italicButton.setToolTipText(PeaProperties.getBundle().getString("italic"));
		italicButton.setFont(new Font(Font.SANS_SERIF, Font.ITALIC, 12));
		italicButton.setMargin(new Insets(2,5,2,5));//top-left-bottom-right
		buttonMenuPanel.add(italicButton);

		JButton underlineButton = new JButton();
		underlineButton.setAction(new StyledEditorKit.UnderlineAction());
		underlineButton.setText(PeaProperties.getBundle().getString("underline_mnemonic"));
		underlineButton.setToolTipText(PeaProperties.getBundle().getString("underline"));
		underlineButton.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		underlineButton.setMargin(new Insets(2,5,2,5));//top-left-bottom-right
		underlineButton.setMaximumSize(underlineButton.getPreferredSize());//new Dimension(30,30));
		buttonMenuPanel.add(underlineButton);

		contentPane.add(buttonMenuPanel);

		// tab are added in PsDialogEditor startDecryption
		tabPane = new JTabbedPane();
		contentPane.add(tabPane);

		this.setLocation(location);
		//System.out.println(UIManager.getLookAndFeel().getName());
		// some L&Fs need more space to display the button verticalBar
		String lfs = UIManager.getLookAndFeel().getName();
		if (lfs.equals("Metal")){
			this.setMinimumSize(new Dimension(450, 500));
		} else if (lfs.equals("Nimbus")){
			this.setMinimumSize(new Dimension(600, 500));
		} else if (lfs.contains("Motif")){
			this.setMinimumSize(new Dimension(600, 500));
		} else if (lfs.startsWith("GTK")){
			this.setMinimumSize(new Dimension(500, 500));
		} else if (lfs.equals("Windows")){
			this.setMinimumSize(new Dimension(500, 500));
		} else {
			this.setMinimumSize(new Dimension(450, 500));
		}
	}
	/**
	 * Get the current instance of ContentUtilities or create a
	 * new one if no instance exists - there is always only
	 * one instance. 
	 * 
	 * @param pswDialog		the associated password dialog
	 * @param loc			the location on screen 
	 * 
	 * @return				the instance of ContentUtilities
	 */
	public static LockFrameEditor getInstance(EditorControl pswDialog, Point loc) {
		LockFrameEditor frameInstance = null; 
		if (frame == null) {
			frameInstance = new LockFrameEditor(pswDialog, loc);
		} else {
			frameInstance = frame;
		}
		return frameInstance;
	}


	// for testing
	public static void main(String[] args) {
		PswDialogView.setUI(false);
		// customize button margin: Done in CalendarControl
		UIManager.put("Button.contentMargins", new Insets( 5,9,5,9) );		

		LockFrameEditor lfe = LockFrameEditor.getInstance(null, new Point(100,100));//CalendarControl pswDialog, Point loc)
		lfe.setVisible(true);
		
	}

	/**
	 * Create a new tab for the given file name and add
	 * it to the JTabbedPane. Unless the tab gets the focus,
	 * the content of the file is not decrypted and displayed. 
	 * 
	 * @param fileName	the name of the text file
	 */
	public void createNewTab(String fileName){
		
		if (new File(fileName).isFile()) {
		
			@SuppressWarnings("unused")
			TabComponent tab = new TabComponent(tabPane, fileName);
		}
	}
	
 
	/**
	 * pack the frame if new components were added or removed
	 */
	public static void packFrame(){
		frame.pack();
	}

	
	/**
	 * Manually add ChangeListener after all tab were added, 
	 * set focus to first tab
	 */
	public void setChangeListener(){

		if (tabPane.getTabCount() > 0) {
			if (tabPane.getSelectedComponent() != null) {
				changeFocus((TabComponent) tabPane.getSelectedComponent());
			} else {
				tabPane.setSelectedIndex(0);
				changeFocus((TabComponent) tabPane.getSelectedComponent());
			}
		}
		tabPane.addChangeListener(this);
	}
	

	/**
	 * Check if the text of the last selected tab
	 * was modified and not saved manually and ask 
	 * to save the text in this case. 
	 */
	public void checkUnsavedTextChanges() {
		if (docChangeUnsaved == true) {
			int n = JOptionPane.showConfirmDialog(
				    this,
				    PeaProperties.getBundle().getString("save_modifications"),//"Save changes?",
				    PeaProperties.getBundle().getString("unsaved_modifications"),//"Unsaved Changes",
				    JOptionPane.YES_NO_OPTION);
			if (n == JOptionPane.YES_OPTION) {
				byte[] plainBytes = getPlainBytes();
				ContentUtilities.saveContent(null, plainBytes, lastFocusTab.getFileName());////((TabComponent) tabPane.getSelectedComponent()).getFileName());
				docChangeUnsaved = false;
			}
		}
	}
	
	/**
	 * If a user sets the focus on a new tab manually
	 * or if the focus is set by the program:
	 * Change the title of the new selected tab and
	 * - set EditorKit
	 * - add MouzseListener for CopyCutPastePopup
	 * - add DocumentListener to register unsaved changes
	 * - add Redo-Undo-Manager
	 * - register KeyStrokes
	 * and remove all this from the previously focused tab
	 * 
	 * @param newFocusTab		the tab that should get the focus
	 */
	private void changeFocus(TabComponent newFocusTab) {
		
		if (tabPane.getTabCount() == 0 ){
			System.err.println("tabPane is empty");
			return;
		}

		// Handle the previously selected tab: 
		if (	lastFocusTab != null 								// exists
				&& (tabPane.indexOfComponent(lastFocusTab) != -1)	// is in tabPane
				&& lastFocusTab.getFileName() != null) {			// was not closed before
				
			// ask to save text if text was changed and not saved manually:
			checkUnsavedTextChanges();
			
			// set no image and gray background: 
			lastFocusTab.setTabTitle(false);
			
			// get the associated JTextPane:
			JTextPane oldTextPane = lastFocusTab.getTextPane();
			
			// remove DocumentListener:
			oldTextPane.getDocument().removeDocumentListener(this);
			
			// remove Cut-Copy-Paste.Menue: 
			oldTextPane.removeMouseListener(lastFocusTab);

			// remove redo and undo manager:
			oldTextPane.getDocument().removeUndoableEditListener(manager);
			
			oldTextPane.setText("");
			   
			//remove keyStroke: STRG + R, STRG + W
			oldTextPane.unregisterKeyboardAction(KeyStroke.getKeyStroke(
			            KeyEvent.VK_R, InputEvent.CTRL_MASK));
			oldTextPane.unregisterKeyboardAction(KeyStroke.getKeyStroke(
			            KeyEvent.VK_W, InputEvent.CTRL_MASK));	
		}

		// get the JTextPane of the new tab:
		JTextPane textPane = newFocusTab.getTextPane();		
		
		// set EditorKit:
		kit = new RTFEditorKit();
		textPane.setEditorKit(kit);
		
		// set the text
		String errorMessage = newFocusTab.setText();
		if (errorMessage != null) { // decryption failed
			
			System.err.println("decryption failed for new focused tab: " + newFocusTab.getFileName()
					+ ", " + errorMessage);
			docChangeUnsaved = false;

			// remove file name from list:
			ArrayList<String> list = PeaControl.getAvailableFileNames();
			if (list.contains(newFocusTab.getFileName())) {
				list.remove(newFocusTab.getFileName());
				PeaControl.setAvailableFileNames(list);
			}
			
			try{
				// remove DocumentListener:
				textPane.getDocument().removeDocumentListener(this);
				
				// remove Cut-Copy-Paste.Menue: 
				if (newFocusTab != null) {
					textPane.removeMouseListener(newFocusTab);
				}

				// remove redo and undo manager:
				textPane.getDocument().removeUndoableEditListener(manager);
				   
				//remove keyStroke: STRG + R, STRG + W
				textPane.unregisterKeyboardAction(KeyStroke.getKeyStroke(
				            KeyEvent.VK_R, InputEvent.CTRL_MASK));
				textPane.unregisterKeyboardAction(KeyStroke.getKeyStroke(
				            KeyEvent.VK_W, InputEvent.CTRL_MASK));	
			} catch (Exception e) {
				System.err.println(e.getMessage());
			}

			// delete the file name: file should never be overwritten
			newFocusTab.setFileName(null);
			
			// get the index of the new tab to set the focus:
			int index = tabPane.indexOfComponent(newFocusTab);
			if (index != -1) { // not in tabPane
				// Handle new selection of tab first: if possible, set to lastFocusTab
				if (lastFocusTab != null 
						&& (tabPane.indexOfComponent(lastFocusTab) != -1)
						&& lastFocusTab.getFileName() != null) {
					tabPane.setSelectedComponent(lastFocusTab);
				} else {
					// set focus to first tab
					if (tabPane.getTabCount() > 0) {
						tabPane.setSelectedComponent(tabPane.getComponentAt(0));
					}
				}
				// remove the tab (which is not selected now)
				if (newFocusTab != null &&
						tabPane.getTabCount() > 0 && 
						(tabPane.indexOfComponent(newFocusTab) != -1)){
					try { // if there is no valid tab left, this will throw an IndexOutOfBoundException
						tabPane.remove(newFocusTab);
					} catch (IndexOutOfBoundsException e) {
						System.err.println(e.toString() + 
								", removing tab failed: " + newFocusTab.getFileName()
								+ ", " + e.getMessage());
					}
					
				} 
			}			
			// do not continue, everything is handled by the new selection
			// and its call to changeFocus
			return;
		}

		
		// Cut-Copy-Paste.Menue: 
		textPane.addMouseListener(newFocusTab);
		// Undo/Redo: key and menu
		manager = new UndoManager();
		UndoAction undoAction = new UndoAction(manager);
		RedoAction redoAction = new RedoAction(manager);
		textPane.getDocument().addUndoableEditListener(manager);
		   
		//keyStroke: STRG + R, STRG + W
		textPane.registerKeyboardAction(undoAction, KeyStroke.getKeyStroke(
		            KeyEvent.VK_R, InputEvent.CTRL_MASK), JComponent.WHEN_FOCUSED);
		textPane.registerKeyboardAction(redoAction, KeyStroke.getKeyStroke(
		            KeyEvent.VK_W, InputEvent.CTRL_MASK), JComponent.WHEN_FOCUSED);
		

		// add listener
		textPane.getDocument().addDocumentListener(this);		

		// set tab title with pea icon and green background
		newFocusTab.setTabTitle(true); // true = icon, green

		// set the component as lastFocusTab
		lastFocusTab = newFocusTab;

		// DocumentListener should not fire event for text setting
		docChangeUnsaved = false;
	}
	
	
	//-------------------------override-----------------------------
	// ChangeListener: event if selection changes: 
	// there are some cases where getSelectedIndex() 
	// and getSelectedComponen() are not the same!!!
	@Override
	public void stateChanged(ChangeEvent arg0) {	
		
		try { 
			// set the wait cursor: 
			this.setCursor(CursorManager.getWaitCursor());

			if (tabPane.getSelectedComponent() == null) {
				// no tab selected
				if (tabPane.getTabCount() > 0){
						//&& tabPane.getComponent(0) instanceof TabComponent) { // not TabContainer
					// set focus on first tab
					TabComponent newFocus = (TabComponent) tabPane.getComponentAt(0);
					if (newFocus != null &&
							tabPane.indexOfComponent(newFocus) != -1){
						tabPane.setSelectedComponent(newFocus);
					}

				} else {
					// no tab
					System.err.println("No tab to set focus...");				
				}
			} else {
				changeFocus((TabComponent) tabPane.getSelectedComponent());
			}						
		} finally {
			// reset the cursor:
	    	this.setCursor(CursorManager.getDefaultCursor());
		} 
	}
	
	// DocumentListener
	@Override
	public void changedUpdate(DocumentEvent arg0) {}
	@Override
	public void insertUpdate(DocumentEvent arg0) {
		docChangeUnsaved = true;
	}
	@Override
	public void removeUpdate(DocumentEvent arg0) {
		docChangeUnsaved = true;	
	}
	
	// ContentUtilities
	protected void windowClosingCommands() {
		
		// ask to save file names in path file
		int len = PeaControl.getAvailableFileNames().size();
		for (int i = 0; i < len; i++) {
			//PswDialogBase.setEncryptedFileName(PswDialogBase.getAvailableFileNames().get(i));
			PathFileManager.pathFileCheck(PeaControl.getAvailableFileNames().get(i));
		}
		
		// ask to save unsaved modifications of the text
		checkUnsavedTextChanges();

		// clear the text
		if (tabPane.getSelectedComponent() != null) {
			StyledDocument doc = ((TabComponent) tabPane.getSelectedComponent()).getTextPane().getStyledDocument();
			try {
				doc.remove(0, doc.getLength() );
			} catch (BadLocationException e) {
				System.err.println("LockFrameCalendar: " + e);
				e.printStackTrace();
			}			
		}
		frame.dispose();
		// clear the key
		CipherStuff.getInstance().getSessionKeyCrypt().clearKeys();
		System.exit(0);			
	}

	/**
	 * Get plain text as byte array 
	 * of current text pane
	 * 
	 * @return	the plain text as byte array
	 */
	public byte[] getPlainBytes() {

		JTextPane textPane = lastFocusTab.getTextPane();//((TabComponent) tabPane.getSelectedComponent()).getTextPane();
		DefaultStyledDocument doc = (DefaultStyledDocument) textPane.getStyledDocument();
		if (doc == null) {
			System.err.println("no document");
		}
		byte[] plainBytes = null;

		// UndoListener can not be serialized.. 
		textPane.getDocument().removeUndoableEditListener(manager);
		plainBytes = Converter.serialize(doc);
	    textPane.getDocument().addUndoableEditListener(manager);
		textPane.getDocument().addDocumentListener(this);

		return plainBytes;
	}
	@Override
	public void windowClosing(WindowEvent arg0) { // by X-Button
		windowClosingCommands();
	}
	@Override
	public void windowActivated(WindowEvent arg0) {}
	@Override
	public void windowClosed(WindowEvent arg0) {}
	@Override
	public void windowDeactivated(WindowEvent arg0) {}
	@Override
	public void windowDeiconified(WindowEvent arg0) {}
	@Override
	public void windowIconified(WindowEvent arg0) {}
	@Override
	public void windowOpened(WindowEvent arg0) {}

	
	
	//--------------------------------------------------------------------------
	// Getter & setter
	
	/**
	 * Set the text for the currently selected tab
	 * as StyledDocument
	 * 
	 * @param textChars		the text as an array of characters
	 */
	protected void setChars( char[] textChars ) {

		//StyledDocument doc = textPane.getStyledDocument();
		StyledDocument doc = ((TabComponent) tabPane.getSelectedComponent()).getTextPane().getStyledDocument();
		try {
			doc.insertString(0, new String(textChars), null );
		} catch (BadLocationException e) {
			System.err.println("LockFrameCalendar: " + e);
			e.printStackTrace();
		}		
	}

	/**
	 * Get the currently focused JTextPane 
	 * 
	 * @return	the currently focused JTextPane 
	 */
	protected static JTextPane getFocusedTextPane(){
		return frame.lastFocusTab.getTextPane();
	}
	/**
	 * Get the currently selected tab
	 * 
	 * @return	the currently selected instance of TabComponent
	 */
	protected static TabComponent getCurrentTab(){
		return frame.lastFocusTab;
	}
	
	/**
	 * Get the file names of all currently available tabs
	 * 
	 * @return	the file names as an array of Strings
	 */
	protected static String[] getFileNames(){

		int len = tabPane.getTabCount();
		String[] fileNames = new String[len];
		for (int i = 0; i < len; i++) {
			fileNames[i] = ( (TabComponent) tabPane.getComponentAt(i)).getFileName();
		}
		return fileNames;
	}

	/**
	 * Set value for unsaved changes of the currently
	 * selected tabs JTextPane. 
	 * 
	 * @param docChangeUnsaved the docChangeUnsaved to set
	 */
	public static void setDocChangeUnsaved(boolean docChangeUnsaved) {
		LockFrameEditor.docChangeUnsaved = docChangeUnsaved;
	}
	

	//----------------------------------------------------------------------------------------
	// classes: 
	
	// The Undo action
	private class UndoAction extends AbstractAction {
		private static final long serialVersionUID = 1L;

		public UndoAction(UndoManager manager) {
	      this.manager = manager;
	    }
	    public void actionPerformed(ActionEvent evt) {
	    	
	    	try {
	  	    	if (manager.canUndo()) {
	  	        manager.undo();
	  	    } else {
		    	JOptionPane.showMessageDialog(frame,
		    			"Cannot undo.");
		    	System.err.println("LockFrameCalendar: undo action failed");
		    	}
	  	    } catch (CannotUndoException e) {
	  	        Toolkit.getDefaultToolkit().beep();
	  	    }
	    	
	    }
	    private UndoManager manager;
	}
	// The Redo action
	private class RedoAction extends AbstractAction {
		private static final long serialVersionUID = 1L;

		public RedoAction(UndoManager manager) {
	      this.manager = manager;
	    }
	    public void actionPerformed(ActionEvent evt) {
	      try {
	    	  if (manager.canRedo()){
	    		  manager.redo();
	    	  } else {
	    		  System.err.println("LockFrameCalendar: redo action failed");
	    	  }
	      } catch (CannotRedoException e) {
	        Toolkit.getDefaultToolkit().beep();
	      }
	    }
	    private UndoManager manager;
	}
	
    public AccessibleContext getAccessibleContext() {
        if (accessibleContext == null) {
            accessibleContext = new AccessibleLockFrameEditor();
        }
        return accessibleContext;
    }

    protected class AccessibleLockFrameEditor extends AccessibleJFrame {
        //Inherit everything, override nothing.
    }
}