package cologne.eck.note_pea;

/*
 * Peafactory - Production of Password Encryption Archives
 * Copyright (C) 2015  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */

/**
 * Menu of main frame of editor pea.  
 */

import java.awt.*;
import java.awt.event.*;
import java.io.*;

import javax.accessibility.AccessibleContext;
import javax.swing.*;
import javax.swing.filechooser.FileFilter;
import javax.swing.text.*;
import javax.swing.text.rtf.RTFEditorKit;

import cologne.eck.all_peas.control.ContentUtilities;
import cologne.eck.all_peas.control.PeaControl;
import cologne.eck.all_peas.data.PeaProperties;
import cologne.eck.all_peas.gui.CursorManager;
import cologne.eck.all_peas.gui.HelpMenu;
import cologne.eck.all_peas.gui.PeaDialog;
import cologne.eck.all_peas.gui.PeaFileChooser;
import cologne.eck.all_peas.gui.PeaMenuBar;
import cologne.eck.all_peas.gui.PswDialogView;
import cologne.eck.tools.Converter;
import cologne.eck.tools.ReadResources;
import cologne.eck.tools.UnexpectedValueException;
import cologne.eck.tools.WriteResources;
import cologne.eck.tools.Zeroizer;
import settings.PeaSettings;

@SuppressWarnings("serial")
final class LockFrameEditorMenu extends PeaMenuBar implements ActionListener {

	// private static ResourceBundle PeaProperties.getBundle();
	private static JFrame lockFrame;// = (JFrame) PeaControl.getDialog().getLockFrame();

	public LockFrameEditorMenu() {

		// PeaProperties.getBundle() = PeaProperties.getBundle();
		// setBorder(new EtchedBorder() );

		lockFrame = (JFrame) PeaControl.getDialog().getLockFrame();
		int fontSize = PswDialogView.getFontsize();

		JMenu fileMenu = new JMenu(PeaProperties.getBundle().getString("file"));// File");//\u00FC");
		fileMenu.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, fontSize));
		// fileMenu.setMnemonic(KeyEvent.VK_F);
		fileMenu.setMnemonic(KeyStroke.getKeyStroke(PeaProperties.getBundle().getString("file_mnemonic")).getKeyCode());
		this.add(fileMenu);

		JMenuItem saveItem = new JMenuItem(PeaProperties.getBundle().getString("save_encrypted"));// save (encrypted)");
		saveItem.setActionCommand("save");
		saveItem.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, fontSize));
		saveItem.setMnemonic(KeyStroke.getKeyStroke(PeaProperties.getBundle().getString("save_mnemonic")).getKeyCode());
		saveItem.addActionListener(this);
		fileMenu.add(saveItem);

		JMenuItem saveAsItem = new JMenuItem(PeaProperties.getBundle().getString("save_as"));// "Save as...");
		saveAsItem.setActionCommand("saveAs");
		saveAsItem.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, fontSize));
		saveAsItem.setMnemonic(
				KeyStroke.getKeyStroke(PeaProperties.getBundle().getString("save_as_mnemonic")).getKeyCode());
		saveAsItem.addActionListener(this);
		fileMenu.add(saveAsItem);

		JMenuItem openItem = new JMenuItem(PeaProperties.getBundle().getString("import_file"));// "open file (txt, rtf
																								// or lock file)");
		openItem.setActionCommand("open");
		openItem.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, fontSize));
		openItem.setMnemonic(
				KeyStroke.getKeyStroke(PeaProperties.getBundle().getString("import_mnemonic")).getKeyCode());
		openItem.addActionListener(this);
		fileMenu.add(openItem);

		if (PeaSettings.getPswGenerator() != null) {
			JMenuItem randomPswItem = new JMenuItem(PeaProperties.getBundle().getString("generate_random_password"));// "Generate
																														// random
																														// password");
			randomPswItem.setActionCommand("randomPsw");
			randomPswItem.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, fontSize));
			randomPswItem.setMnemonic(
					KeyStroke.getKeyStroke(PeaProperties.getBundle().getString("random_psw_mnemonic")).getKeyCode());
			randomPswItem.addActionListener(this);
			fileMenu.add(randomPswItem);
		}

		JMenuItem pswItem = new JMenuItem(PeaProperties.getBundle().getString("change_password"));// "Change password");
		pswItem.setActionCommand("psw");
		pswItem.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, fontSize));
		pswItem.setMnemonic(
				KeyStroke.getKeyStroke(PeaProperties.getBundle().getString("change_psw_mnemonic")).getKeyCode());
		pswItem.addActionListener(this);
		fileMenu.add(pswItem);

		JMenuItem quitItem = new JMenuItem(PeaProperties.getBundle().getString("quit"));// "Quit");
		quitItem.setActionCommand("quit");
		quitItem.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, fontSize));
		quitItem.setMnemonic(KeyStroke.getKeyStroke(PeaProperties.getBundle().getString("quit_mnemonic")).getKeyCode());
		quitItem.addActionListener(this);
		fileMenu.add(quitItem);

		JMenu editMenu = new JMenu(PeaProperties.getBundle().getString("edit"));// "Edit");
		editMenu.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, fontSize));
		editMenu.setMnemonic(KeyStroke.getKeyStroke(PeaProperties.getBundle().getString("edit_mnemonic")).getKeyCode());
		this.add(editMenu);

		JMenuItem undoItem = new JMenuItem(PeaProperties.getBundle().getString("undo"));// "Undo");
		undoItem.setActionCommand("undo");
		undoItem.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, fontSize));
		undoItem.setMnemonic(KeyStroke.getKeyStroke(PeaProperties.getBundle().getString("undo_mnemonic")).getKeyCode());
		undoItem.addActionListener(this);
		editMenu.add(undoItem);

		JMenuItem redoItem = new JMenuItem(PeaProperties.getBundle().getString("redo"));// "Redo");
		redoItem.setActionCommand("redo");
		redoItem.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, fontSize));
		redoItem.setMnemonic(KeyStroke.getKeyStroke(PeaProperties.getBundle().getString("redo_mnemonic")).getKeyCode());
		redoItem.addActionListener(this);
		editMenu.add(redoItem);

		JMenuItem cutItem = new JMenuItem(PeaProperties.getBundle().getString("cut"));// "Cut");
		cutItem.setActionCommand("cut");
		cutItem.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, fontSize));
		cutItem.setMnemonic(KeyStroke.getKeyStroke(PeaProperties.getBundle().getString("cut_mnemonic")).getKeyCode());
		cutItem.addActionListener(this);
		editMenu.add(cutItem);

		JMenuItem copyItem = new JMenuItem(PeaProperties.getBundle().getString("copy"));// "Copy");
		copyItem.setActionCommand("copy");
		copyItem.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, fontSize));
		copyItem.setMnemonic(KeyStroke.getKeyStroke(PeaProperties.getBundle().getString("copy_mnemonic")).getKeyCode());
		copyItem.addActionListener(this);
		editMenu.add(copyItem);

		JMenuItem pasteItem = new JMenuItem(PeaProperties.getBundle().getString("paste"));// "Paste");
		pasteItem.setActionCommand("paste");
		pasteItem.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, fontSize));
		pasteItem.setMnemonic(
				KeyStroke.getKeyStroke(PeaProperties.getBundle().getString("paste_mnemonic")).getKeyCode());
		pasteItem.addActionListener(this);
		editMenu.add(pasteItem);

		JMenuItem selectAllItem = new JMenuItem(PeaProperties.getBundle().getString("select_all"));// "Select All");
		selectAllItem.setActionCommand("selectAll");
		selectAllItem.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, fontSize));
		selectAllItem.setMnemonic(
				KeyStroke.getKeyStroke(PeaProperties.getBundle().getString("select_all_mnemonic")).getKeyCode());
		selectAllItem.addActionListener(this);
		editMenu.add(selectAllItem);

		JMenu formatMenu = new JMenu(PeaProperties.getBundle().getString("format"));// "Format");
		formatMenu.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, fontSize));
		formatMenu.setMnemonic(
				KeyStroke.getKeyStroke(PeaProperties.getBundle().getString("format_mnemonic")).getKeyCode());
		this.add(formatMenu);

		JMenu fontMenu = new JMenu(PeaProperties.getBundle().getString("font"));// "Font");
		fontMenu.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, fontSize));
		fontMenu.setMnemonic(KeyStroke.getKeyStroke(PeaProperties.getBundle().getString("font_mnemonic")).getKeyCode());
		formatMenu.add(fontMenu);

		String fontNames[] = { "SERIF", "SANS_SERIF", "MONOSPACED" };
		for (String font : fontNames) {
			fontMenu.add(new StyledEditorKit.FontFamilyAction(font, font));
		}

		JMenu fontStyleMenu = new JMenu(PeaProperties.getBundle().getString("font_style"));// "Font style");
		fontStyleMenu.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, fontSize));
		fontStyleMenu.setMnemonic(
				KeyStroke.getKeyStroke(PeaProperties.getBundle().getString("style_mnemonic")).getKeyCode());
		formatMenu.add(fontStyleMenu);

		Action action = new StyledEditorKit.BoldAction();
		action.putValue(Action.NAME, PeaProperties.getBundle().getString("bold"));// "Bold");
		fontStyleMenu.add(action);

		action = new StyledEditorKit.ItalicAction();
		action.putValue(Action.NAME, PeaProperties.getBundle().getString("italic"));// "Italic");
		fontStyleMenu.add(action);

		action = new StyledEditorKit.UnderlineAction();
		action.putValue(Action.NAME, PeaProperties.getBundle().getString("underline"));// "Underline");
		fontStyleMenu.add(action);

		JMenu alignmentMenu = new JMenu(PeaProperties.getBundle().getString("alignment"));// "Font style");
		alignmentMenu.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, fontSize));
		alignmentMenu.setMnemonic(
				KeyStroke.getKeyStroke(PeaProperties.getBundle().getString("alignment_mnemonic")).getKeyCode());
		formatMenu.add(alignmentMenu);

		Action alignmentAction = new StyledEditorKit.AlignmentAction(PeaProperties.getBundle().getString("left"), 0);
		alignmentMenu.add(alignmentAction);

		alignmentAction = new StyledEditorKit.AlignmentAction(PeaProperties.getBundle().getString("center"), 1);
		alignmentMenu.add(alignmentAction);

		alignmentAction = new StyledEditorKit.AlignmentAction(PeaProperties.getBundle().getString("right"), 2);
		alignmentMenu.add(alignmentAction);

		JMenu fontSizeMenu = new JMenu(PeaProperties.getBundle().getString("font_size"));// "Font size");
		fontSizeMenu.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, fontSize));
		fontSizeMenu
				.setMnemonic(KeyStroke.getKeyStroke(PeaProperties.getBundle().getString("size_mnemonic")).getKeyCode());
		formatMenu.add(fontSizeMenu);

		for (int i = 6; i < 25; i++) {
			fontSizeMenu.add(new StyledEditorKit.FontSizeAction("" + i, i));
		}

		JMenuItem fontColorMenu = new JMenuItem(PeaProperties.getBundle().getString("font_color"));
		fontColorMenu.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, fontSize));
		fontColorMenu.setMnemonic(
				KeyStroke.getKeyStroke(PeaProperties.getBundle().getString("color_mnemonic")).getKeyCode());
		fontColorMenu.addActionListener(this);
		fontColorMenu.setActionCommand("fontColor");
		formatMenu.add(fontColorMenu);

		JMenu helpMenu = new HelpMenu();
		this.add(helpMenu);

		this.setMaximumSize(this.getPreferredSize());
	}

	@Override
	public void actionPerformed(ActionEvent ape) {

		String command = ape.getActionCommand();

		if (command.equals("save")) { // saves current content in current file
			try {
				byte[] plainBytes = ((LockFrameEditor) PeaControl.getDialog().getLockFrame()).getPlainBytes();
				String fileName = LockFrameEditor.getCurrentTab().getFileName();

				ContentUtilities.saveContent(null, plainBytes, fileName);// gPswDialogBase.getEncryptedFileName());

				LockFrameEditor.setDocChangeUnsaved(false);
			} catch (Exception e) {
				System.err.println("EditorLockFrame Menu (save): \n" + e.toString() + ", " + e.getMessage());
				PeaDialog.showMessage(lockFrame, PeaProperties.getBundle().getString("unexpected_error"),
						PeaProperties.getBundle().getString("error"), 0 // ERROR_MESSAGE
				);

				/*
				 * JOptionPane.showMessageDialog( PeaControl.getDialog().getLockFrame(),
				 * PeaProperties.getBundle().getString("unexpected_error"),
				 * PeaProperties.getBundle().getString("error"), JOptionPane.ERROR_MESSAGE);
				 */
			}
		} else if (command.equals("saveAs")) {

			Object[] possibleOptions = { PeaProperties.getBundle().getString("simple_plaintext_file"), // "A. save as
																										// simple plain
																										// text file
																										// (unencrypted)",
					PeaProperties.getBundle().getString("plaintext_file_rtf") };// "B. save as plain text in rich text
																				// format (unencrypted)"};
			Object selectedValue = JOptionPane.showInputDialog(this,
					PeaProperties.getBundle().getString("choose_how_to_save"), // "Choose how you want to save the file:
																				// \n",
					PeaProperties.getBundle().getString("choose"), // "Choose...",
					JOptionPane.PLAIN_MESSAGE, null, possibleOptions, possibleOptions[0]);
			FileFilter filter = null;
			String fileName = null;
			String filePath = null;

			if (selectedValue == null) { // cancel
				return;
			} else if (((String) selectedValue).startsWith("A")) {
				filter = new TxtFilter();
			} else if (((String) selectedValue).startsWith("B")) {
				filter = new RtfFilter();
			}
			final PeaFileChooser fc = new PeaFileChooser(this);
			fc.setFileFilter(filter);
			int returnVal = fc.showSaveDialog((JFrame) PeaControl.getDialog().getLockFrame());
			if (returnVal == PeaFileChooser.APPROVE_OPTION) {
				fileName = fc.getSelectedFile().getName();
				filePath = fc.getSelectedFile().getPath();

			} else { // cancel option
				return;
			}

			// WriteRessources...
			if (filter instanceof TxtFilter) {
				String text = null;
				try {
					JTextPane textPane = LockFrameEditor.getFocusedTextPane();
					text = textPane.getDocument().getText(0, textPane.getDocument().getLength());
				} catch (BadLocationException e) {
					System.err.println("EditorLockFrame Menu (Save as txt): \n" + e.toString() + ", " + e.getMessage());
					JOptionPane.showMessageDialog(this,
							// PeaControl.getDialog().getLockFrame(),
							PeaProperties.getBundle().getString("unexpected_error"),
							PeaProperties.getBundle().getString("error"), JOptionPane.ERROR_MESSAGE);
				}
				byte[] textBytes = text.getBytes(PeaProperties.getCharset());// .Converter.chars2bytes(text.toCharArray());
				if (!fileName.endsWith(".txt")) {
					filePath = filePath + ".txt";
				}
				if (new File(filePath).exists()) {
					System.out.println("Warning existing file will be overwritten");
					int n = JOptionPane.showConfirmDialog(this,
							// PeaControl.getDialog().getLockFrame(),
							PeaProperties.getBundle().getString("overwrite_existing_file"),
							PeaProperties.getBundle().getString("warning"), JOptionPane.OK_CANCEL_OPTION,
							JOptionPane.WARNING_MESSAGE);
					if (n != JFileChooser.APPROVE_OPTION) { // cancel, close
						return;
						// System.exit(0);
					}
				}
				boolean success = WriteResources.write(textBytes, filePath, null);
				if (success == false) {
					System.err.println("EditorLockFrame Menu (Save as txt): \n");
					JOptionPane.showMessageDialog(this,
							// PeaControl.getDialog().getLockFrame(),
							PeaProperties.getBundle().getString("read_write_failed"),
							PeaProperties.getBundle().getString("error"), JOptionPane.ERROR_MESSAGE);

				}
			} else if (filter instanceof RtfFilter) {

				if (!filePath.toLowerCase().endsWith("rtf")) {
					filePath = filePath + ".rtf";
				}
				if (new File(filePath).exists()) {
					System.out.println("Warning existing file will be overwritten");
					int n = JOptionPane.showConfirmDialog(this,
							// PeaControl.getDialog().getLockFrame(),
							PeaProperties.getBundle().getString("overwrite_existing_file"),
							PeaProperties.getBundle().getString("warning"), JOptionPane.OK_CANCEL_OPTION,
							JOptionPane.WARNING_MESSAGE);
					if (n != JFileChooser.APPROVE_OPTION) { // cancel, close
						return;
						// System.exit(0);
					}
				}
				JTextPane textPane = LockFrameEditor.getFocusedTextPane();
				StyledDocument doc = textPane.getStyledDocument();
				RTFEditorKit kit = new RTFEditorKit();
				BufferedOutputStream out;
				try {
					out = new BufferedOutputStream(new FileOutputStream(filePath));
					kit.write(out, doc, doc.getStartPosition().getOffset(), doc.getLength());
					out.close();
				} catch (FileNotFoundException e) {
					System.err.println("EditorLockFrame Menu (Save rtf): \n" + e.toString() + ", " + e.getMessage());
					JOptionPane.showMessageDialog(this,
							// PeaControl.getDialog().getLockFrame(),
							PeaProperties.getBundle().getString("read_write_failed") + "\n"
									+ PeaProperties.getBundle().getString("file_not_found") + ": \n" + filePath,
							PeaProperties.getBundle().getString("error"), JOptionPane.ERROR_MESSAGE);
				} catch (IOException e) {
					System.err.println("EditorLockFrame Menu (Save rtf): \n" + e.toString() + ", " + e.getMessage());
					JOptionPane.showMessageDialog(this,
							// PeaControl.getDialog().getLockFrame(),
							PeaProperties.getBundle().getString("read_write_failed") + "\n"
									+ PeaProperties.getBundle().getString("unexpected_error") + "\n " + e.getMessage()
									+ ": \n" + filePath,
							PeaProperties.getBundle().getString("error"), JOptionPane.ERROR_MESSAGE);
				} catch (BadLocationException e) {
					System.err.println("EditorLockFrame Menu (Save rtf): \n" + e.toString() + ", " + e.getMessage());
					JOptionPane.showMessageDialog(this,
							// PeaControl.getDialog().getLockFrame(),
							PeaProperties.getBundle().getString("read_write_failed") + "\n"
									+ PeaProperties.getBundle().getString("unexpected_error") + "\n " + e.getMessage()
									+ ": \n" + filePath,
							PeaProperties.getBundle().getString("error"), JOptionPane.ERROR_MESSAGE);
				}
			}

		} else if (command.equals("open")) { // current content get lost if not saved

			Object[] possibleOptions = { // "open encrypted lock file",
					PeaProperties.getBundle().getString("open_plaintext_file"),
					PeaProperties.getBundle().getString("open_plaintext_rtf_file"),
					// PeaProperties.getBundle().getString("open_html_file")
			};
			Object selectedValue = JOptionPane.showInputDialog(this,
					PeaProperties.getBundle().getString("open_what_kind_of_file"), // "Open what kind of file? \n",
					null, JOptionPane.PLAIN_MESSAGE, null, possibleOptions, possibleOptions[0]);
			FileFilter filter = null;
			File file = null;
			String fileName = null;
			if (selectedValue == null) { // cancel
				return;
			} else if (selectedValue.equals(possibleOptions[0])) {
				filter = new TxtFilter();
			} else if (selectedValue.equals(possibleOptions[1])) {
				filter = new RtfFilter();
			} /*
				 * else if ( selectedValue.equals(possibleOptions[2]) ) { filter = new
				 * FileNameExtensionFilter("Html files", "html", "HTML"); }
				 */
			final PeaFileChooser fc = new PeaFileChooser(this);
			fc.setFileFilter(filter);
			int returnVal = fc.showOpenDialog(this);
			// PeaControl.getDialog().getLockFrame());
			if (returnVal == PeaFileChooser.APPROVE_OPTION) {
				file = fc.getSelectedFile();
				try {
					fileName = file.getCanonicalPath();
				} catch (IOException e) {
					System.err.println(
							"EditorLockFrame Menu (Open canonicalpath): \n" + e.toString() + ", " + e.getMessage());
					JOptionPane.showMessageDialog(this,
							// PeaControl.getDialog().getLockFrame(),
							PeaProperties.getBundle().getString("unexpected_error"),
							PeaProperties.getBundle().getString("error"), JOptionPane.ERROR_MESSAGE);

				}
			} else { // cancel option
				return;
			}
			try {
				// warning: long execution time
				if (file.length() > 1048576) { // 1 MiB
					JOptionPane.showMessageDialog(this,
							// PeaControl.getDialog().getLockFrame(),
							"Large file may cause long execution time. ",
							PeaProperties.getBundle().getString("warning"), JOptionPane.WARNING_MESSAGE);
					// set the wait cursor:
					if (lockFrame == null) {
						lockFrame = (JFrame) PeaControl.getDialog().getLockFrame();
					}
					lockFrame.setCursor(CursorManager.getWaitCursor());
				}
				// avoid too large files to encrypt
				if (file.length() > 8388608) { // 8 MiB limit to avoid memory errors
					JOptionPane.showMessageDialog(this,
							// PeaControl.getDialog().getLockFrame(),
							"The selected file is too large. ", PeaProperties.getBundle().getString("error"),
							JOptionPane.ERROR_MESSAGE);
					// reset the cursor:
					if (lockFrame == null) {
						lockFrame = (JFrame) PeaControl.getDialog().getLockFrame();
					}
					lockFrame.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
					return;
				}
				JTextPane textPane = LockFrameEditor.getFocusedTextPane();
				Document doc = textPane.getDocument();
				int docLen = doc.getLength();

				if (fileName.endsWith(".txt") || fileName.endsWith(".TXT")) {
					byte[] textBytes = ReadResources.readExternFile(fileName);
					if (textBytes != null) {
						char[] textChars = Converter.bytes2chars(textBytes);

						try {
							doc.insertString(docLen, new String(textChars), null);
						} catch (BadLocationException e) {
							System.err.println(
									"EditorLockFrame Menu (Open txt): \n" + e.toString() + ", " + e.getMessage());
							JOptionPane.showMessageDialog(this,
									// PeaControl.getDialog().getLockFrame(),
									PeaProperties.getBundle().getString("unexpected_error") + "\n " + e.getMessage()
											+ ": \n" + fileName,
									PeaProperties.getBundle().getString("error"), JOptionPane.ERROR_MESSAGE);
						}
						Zeroizer.zero(textChars);
					}
				} else if (fileName.endsWith(".rtf") || fileName.endsWith(".RTF")) {

					RTFEditorKit kit = (RTFEditorKit) textPane.getEditorKit();

					FileInputStream in;
					try {
						in = new FileInputStream(fileName);
						kit.read(in, doc, docLen);
						in.close();
					} catch (FileNotFoundException e) {
						System.err
								.println("EditorLockFrame Menu (Open rtf): \n" + e.toString() + ", " + e.getMessage());
						JOptionPane.showMessageDialog(this,
								// PeaControl.getDialog().getLockFrame(),
								PeaProperties.getBundle().getString("file_not_found") + "\n " + e.getMessage() + ": \n"
										+ fileName,
								PeaProperties.getBundle().getString("error"), JOptionPane.ERROR_MESSAGE);
					} catch (IOException e) {
						System.err
								.println("EditorLockFrame Menu (Open rtf): \n" + e.toString() + ", " + e.getMessage());
						JOptionPane.showMessageDialog(this,
								// PeaControl.getDialog().getLockFrame(),
								PeaProperties.getBundle().getString("unsuitable_rtf_file") + "\n " + e.getMessage()
										+ ": \n" + fileName,
								PeaProperties.getBundle().getString("error"), JOptionPane.ERROR_MESSAGE);
					} catch (BadLocationException e) {
						System.err
								.println("EditorLockFrame Menu (Open rtf): \n" + e.toString() + ", " + e.getMessage());
						JOptionPane.showMessageDialog(this,
								// PeaControl.getDialog().getLockFrame(),
								PeaProperties.getBundle().getString("unexpected_error") + "\n " + e.getMessage()
										+ ": \n" + fileName,
								PeaProperties.getBundle().getString("error"), JOptionPane.ERROR_MESSAGE);
					}
				} /*
					 * else if (fileName.endsWith(".html") || fileName.endsWith(".HTML")) {
					 * convertToRTF(fileName); }
					 */
			} finally {
				// reset the cursor:
				if (lockFrame == null) {
					lockFrame = (JFrame) PeaControl.getDialog().getLockFrame();
				}
				lockFrame.setCursor(CursorManager.getDefaultCursor());
			}
		} else if (command.equals("fontColor")) {

			Color color = JColorChooser.showDialog(this,
					// PeaControl.getDialog().getLockFrame(),
					PeaProperties.getBundle().getString("font_color"), Color.BLACK);

			if (color != null) {
				// System.out.println(color);
				// set color
				SimpleAttributeSet attr = new SimpleAttributeSet();
				StyleConstants.setForeground(attr, color);
				JTextPane textPane = LockFrameEditor.getFocusedTextPane();
				textPane.setCharacterAttributes(attr, false);
			}

		} else if (command.equals("randomPsw")) {
			JDialog dialog = PeaSettings.getPswGenerator();
			if (dialog == null) {
				if (PeaProperties.getWorkingMode().equals("-t")) {
					new UnexpectedValueException("dialog", "JDialog", "is null").printDescription();
				}
			}
			dialog.setIconImage(PswDialogView.getImage());
			dialog.setLocationRelativeTo(this);
			dialog.setVisible(true);
			// PeaSettings.getPswGenerator().setVisible(true);

		} else if (command.equals("psw")) {
			// only for current tab!

			String fileName = LockFrameEditor.getCurrentTab().getFileName();
			// String[] fileNames = {fileName};
			// TODO: change password for other files too
			ContentUtilities.changePassword(((LockFrameEditor) PeaControl.getDialog().getLockFrame()).getPlainBytes(),
					fileName, (LockFrameEditor) PeaControl.getDialog().getLockFrame()); // name of file of current text

		} else if (command.equals("quit")) {

			((LockFrameEditor) PeaControl.getDialog().getLockFrame()).checkUnsavedTextChanges();

			JTextPane textPane = LockFrameEditor.getFocusedTextPane();
			StyledDocument doc = textPane.getStyledDocument();
			try {
				doc.remove(0, doc.getLength());
			} catch (BadLocationException e) {
				// don't show message...
				System.err.println("LockFrameEditorMenu (Quit): \n" + "Can't remove text from Textpane, \n"
						+ e.getMessage() + ", " + e.toString());
			}
			System.exit(0);
		} else if (command.equals("undo")) {
			if (LockFrameEditor.manager.canUndo()) {
				LockFrameEditor.manager.undo();
			}
		} else if (command.equals("redo")) {
			if (LockFrameEditor.manager.canRedo()) {
				LockFrameEditor.manager.redo();
			}
		} else if (command.equals("cut")) {
			JTextPane textPane = LockFrameEditor.getFocusedTextPane();
			textPane.cut();
		} else if (command.equals("copy")) {
			JTextPane textPane = LockFrameEditor.getFocusedTextPane();
			textPane.copy();
		} else if (command.equals("paste")) {
			JTextPane textPane = LockFrameEditor.getFocusedTextPane();
			textPane.paste();
			// Clipboard clipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
			// TransferHandler transferHandler =
			// LockFrameCalendar.textPane.getTransferHandler();
			// transferHandler.importData(LockFrameCalendar.textPane,
			// clipboard.getContents(null));
		} else if (command.equals("selectAll")) {
			JTextPane textPane = LockFrameEditor.getFocusedTextPane();
			textPane.selectAll();

		} else {
			System.err.println("invalid command in editor menu: " + command);
		}
	}
	/*
	 * This can only replace the existing content and only works for html 3...
	 * 
	 * private void convertToRTF(String fileName) {
	 * 
	 * String htmlStr = new String( ReadResources.readExternFile(fileName),
	 * PswDialogBase.getCharset());
	 * 
	 * OutputStream os = new ByteArrayOutputStream(); HTMLEditorKit htmlEditorKit =
	 * new HTMLEditorKit(); RTFEditorKit rtfEditorKit = new RTFEditorKit(); String
	 * rtfStr = null;
	 * 
	 * htmlStr = htmlStr.replaceAll("<br.*?>","#NEW_LINE#"); htmlStr =
	 * htmlStr.replaceAll("</p>","#NEW_LINE#"); htmlStr =
	 * htmlStr.replaceAll("</li>","#NEW_LINE#"); htmlStr =
	 * htmlStr.replaceAll("<p.*?>",""); InputStream is = new
	 * ByteArrayInputStream(htmlStr.getBytes()); try { Document doc =
	 * htmlEditorKit.createDefaultDocument(); htmlEditorKit.read(is, doc, 0);
	 * rtfEditorKit .write(os, doc, 0, doc.getLength()); rtfStr = os.toString();
	 * rtfStr = rtfStr.replaceAll("#NEW_LINE#","\\\\par ");
	 * LockFrameCalendar.textPane.setText( rtfStr); is.close(); } catch (IOException
	 * e) { e.printStackTrace(); } catch (BadLocationException e) {
	 * e.printStackTrace(); } }
	 */

	// --------------------------------
	// inner classes
	//
	public class TxtFilter extends FileFilter {
		public boolean accept(File f) {
			return f.isDirectory() || f.getName().toLowerCase().endsWith(".txt")
					|| f.getName().toLowerCase().endsWith(".text");
		}

		public String getDescription() {
			return "text" + " (*.txt)";
		}
	};

	public class RtfFilter extends FileFilter {
		public boolean accept(File f) {
			return f.isDirectory() || f.getName().toLowerCase().endsWith(".rtf");
		}

		public String getDescription() {
			return "rich text format" + " (*.rtf)";
		}
	};

    public AccessibleContext getAccessibleContext() {
        if (accessibleContext == null) {
            accessibleContext = new AccessibleLockFrameEditorMenu();
        }
        return accessibleContext;
    }

    protected class AccessibleLockFrameEditorMenu extends AccessiblePeaMenuBar {
        //Inherit everything, override nothing.
    }
}