package cologne.eck.peafactory.control;

/*
 * Peafactory - Production of Password Encryption Archives
 * Copyright (C) 2015  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */

/**
 * Managing of entries of the jar file (Password Encrypted Archive).
 * The pea should contain only the required classes. 
 */

import java.io.*;
import java.util.ArrayList;
import java.util.jar.*;
import java.util.regex.Matcher;

import org.bouncycastle.crypto.digests.Blake2bDigest;
//import org.bouncycastle.crypto.digests.RIPEMD256Digest;
import org.bouncycastle.crypto.digests.RIPEMD320Digest;
import org.bouncycastle.crypto.digests.SHA384Digest;
import org.bouncycastle.crypto.digests.SHA3Digest;
import org.bouncycastle.crypto.digests.SHA512Digest;
import org.bouncycastle.crypto.digests.SkeinDigest;
import org.bouncycastle.crypto.digests.WhirlpoolDigest;
import org.bouncycastle.crypto.engines.AESEngine;
import org.bouncycastle.crypto.engines.AESFastEngine;
import org.bouncycastle.crypto.engines.SerpentEngine;
import org.bouncycastle.crypto.engines.Shacal2Engine;
import org.bouncycastle.crypto.engines.ThreefishEngine;
import org.bouncycastle.crypto.engines.TwofishEngine;

import cologne.eck.peafactory.crypto.*;
import cologne.eck.peafactory.crypto.kdf.*;
import cologne.eck.peafactory.gui.MainView;
import cologne.eck.tools.WriteResources;





public class CreateJarFile {
	//All JarEntry's names should NOT begin with "/".
	//Directory names must end with a slash '/'
	//All paths must use '/' style slashes, not '\'

	
	public static void createJarFile() { 
		
		DataType dataType = DataType.getCurrentType();

		String currentJarFileName = JarStuff.getJarFileName();
		//System.out.println("CreateJarFile, jarFileName: " + currentJarFileName);
		if (PeaFactory.isBlankPea() == true ||
				(MainView.getOpenedFileName() == null 
				&& !(dataType instanceof FileType) 
				&& !(dataType instanceof ImageType && PeaFactory.isBlankPea() == false))) {
			String dirName = JarStuff.getJarFileName().substring(0, JarStuff.getJarFileName().length() - 4); // extract ".jar"
			currentJarFileName = dirName + File.separator + currentJarFileName;
		}
		currentJarFileName = "peas" + File.separator + currentJarFileName;

		// replace with File.seperator to do some normal file operations
		if (currentJarFileName.contains("/") || currentJarFileName.contains("\\")) {
			currentJarFileName = currentJarFileName.replaceAll("/", File.separator);
			if (currentJarFileName.contains(Matcher.quoteReplacement("\\")) ){
				currentJarFileName = currentJarFileName.replaceAll( Matcher.quoteReplacement("\\"), File.separator  );
			}
			// if last directory not exist: create with all parent directories
			int index1 =  currentJarFileName.lastIndexOf( currentJarFileName.lastIndexOf(File.separator) -1 ) + 1;// if only one exists: return -1
			String lastDir = currentJarFileName.substring( index1 , currentJarFileName.lastIndexOf(File.separator));
			if ( ! new File(lastDir).exists()) {
				if (! new File(lastDir).mkdirs() ) {
					System.err.println("CreateJarFile.java: Can not create directories: " + lastDir);
					System.exit(3);
				}
			}
		}
		// operation on jar-file:		
		File jarFile = new File(currentJarFileName);
		// warn if file already exists
		if ( jarFile.exists() ) {
			System.out.println("Warning: existing jar-archive will be overwritten");
		}
		
		// Name of pea package
		//String pathToPeaClasses = "bin" + File.separator + PeaFactory.getPackagePath() + File.separator;
		String sep = File.separator;
		String pathToAllPeaClasses = "bin" + sep + "cologne" + sep + "eck" + sep 
				+ "all_peas" + sep;
		String pathToToolsClasses = "bin" + sep + "cologne" + sep + "eck" + sep 
				+ "tools" + sep;
		String pathToCryptoClasses = "bin" + sep + "cologne" + sep + "eck" + sep 
				+ "peafactory" + sep + "crypto" + sep;
		String pathToEckCryptoClasses = "bin" + sep + "cologne" + sep + "eck" + sep 
				+ "crypto" + sep;
		String pathToEckClasses = "bin" + sep + "cologne" + sep + "eck" + sep; 
		String pathToBcClasses = "bin" + sep + "org" + sep + "bouncycastle" + sep;

		// Manifest:
		Manifest manifest = new Manifest();
		Attributes att = manifest.getMainAttributes();		
		att.put(Attributes.Name.MANIFEST_VERSION, "1.0");
		if (dataType instanceof ImageType) {
			// works with dir
//			att.put(Attributes.Name.CLASS_PATH, ". lib/pdfbox-2.0.2.jar lib/fontbox-2.0.2.jar lib/commons-logging-1.2.jar");
			// does not work
			//att.put(Attributes.Name.CLASS_PATH, ". lib/* ");
			//att.put(Attributes.Name.CLASS_PATH, ". /lib/* ");
		} else {
			att.put(Attributes.Name.CLASS_PATH, ".");
		}
		att.put(Attributes.Name.MAIN_CLASS, "start.Start");


		//-----------------
		// add the classes
		//
		//File[] tmpClassFiles = new File[156]; // must be enough, we yet do not know the number
		//int index = 0;
		ArrayList <File> classFiles = new ArrayList<File>();

		//==============================================
		// FIXED classes for all peas:
		//==============================================
		
		//===========================
		// peafactory classes:
		classFiles.add( new File("start/Start.class"));
		classFiles.add( new File("settings/PeaSettings.class"));
		
		classFiles.add( new File(pathToAllPeaClasses + "control/PeaControl.class"));	
		classFiles.add( new File(pathToAllPeaClasses + "control/PathFileManager.class"));
		classFiles.add( new File(pathToAllPeaClasses + "control/ContentUtilities.class"));
		
		classFiles.add( new File(pathToAllPeaClasses + "data/PeaProperties.class"));
		classFiles.add( new File(pathToAllPeaClasses + "data/Attachments.class"));
		
		classFiles.add( new File(pathToAllPeaClasses + "files/ExecutionTimeObserver.class"));
		classFiles.add( new File(pathToAllPeaClasses + "files/FileComposer.class"));
		classFiles.add( new File(pathToAllPeaClasses + "files/FileModel.class"));
	//	classFiles.add( new File(pathToAllPeaClasses + "files/ViewMenu.class"));
	//	classFiles.add( new File(pathToAllPeaClasses + "files/ViewMenu$AccessibleViewMenu.class"));
		classFiles.add( new File(pathToAllPeaClasses + "files/FileTypePanel.class"));
		classFiles.add( new File(pathToAllPeaClasses + "files/FileTypePanel$ProgressTask.class")); // inner class
		classFiles.add( new File(pathToAllPeaClasses + "files/FileTypePanel$AccessibleFileTypePanel.class")); // inner class
		
		classFiles.add( new File(pathToAllPeaClasses + "gui/PswDialogView.class"));
		classFiles.add( new File(pathToAllPeaClasses + "gui/PswDialogView$WhileTypingThread.class")); // inner class
		classFiles.add( new File(pathToAllPeaClasses + "gui/PswDialogView$AccessiblePswDialogView.class"));
		classFiles.add( new File(pathToAllPeaClasses + "gui/CharTable.class"));	
		classFiles.add( new File(pathToAllPeaClasses + "gui/CharTable$AccessibleCharTable.class"));			
		classFiles.add( new File(pathToAllPeaClasses + "gui/KeyButton.class"));
		classFiles.add( new File(pathToAllPeaClasses + "gui/KeyButton$AccessibleKeyButton.class"));
		classFiles.add( new File(pathToAllPeaClasses + "gui/PathFileSetting.class"));
		classFiles.add( new File(pathToAllPeaClasses + "gui/PathFileSetting$AccessiblePathFileSetting.class"));
		classFiles.add( new File(pathToAllPeaClasses + "gui/PeaFileChooser.class"));
		classFiles.add( new File(pathToAllPeaClasses + "gui/PeaFileChooser$AccessiblePeaFileChooser.class"));
		classFiles.add( new File(pathToAllPeaClasses + "gui/HelpMenu.class"));
		classFiles.add( new File(pathToAllPeaClasses + "gui/HelpMenu$AccessibleHelpMenu.class"));
		classFiles.add( new File(pathToAllPeaClasses + "gui/PeaMenuBar.class"));
		classFiles.add( new File(pathToAllPeaClasses + "gui/PeaMenuBar$AccessiblePeaMenuBar.class"));
		classFiles.add( new File(pathToAllPeaClasses + "gui/PeaBorderFactory.class"));
		classFiles.add( new File(pathToAllPeaClasses + "gui/NewPasswordDialog.class"));
		classFiles.add( new File(pathToAllPeaClasses + "gui/NewPasswordDialog$AccessibleNewPasswordDialog.class"));
		classFiles.add( new File(pathToAllPeaClasses + "gui/Keyboard.class"));
		classFiles.add( new File(pathToAllPeaClasses + "gui/Keyboard$AccessibleKeyboard.class"));
		classFiles.add( new File(pathToAllPeaClasses + "gui/MultiFileSelectionDialog.class"));
		classFiles.add( new File(pathToAllPeaClasses + "gui/MultiFileSelectionDialog$AccessibleMultiFileSelectionDialog.class"));
		classFiles.add( new File(pathToAllPeaClasses + "gui/CursorManager.class"));
		classFiles.add( new File(pathToAllPeaClasses + "gui/PeaDialog.class"));
		classFiles.add( new File(pathToAllPeaClasses + "gui/IconManager.class"));
		classFiles.add( new File(pathToAllPeaClasses + "gui/ViewMenu.class"));
		classFiles.add( new File(pathToAllPeaClasses + "gui/ViewMenu$AccessibleViewMenu.class"));

		
		classFiles.add( new File(pathToCryptoClasses + "RandomStuff.class"));
		classFiles.add( new File(pathToCryptoClasses + "AuthenticatedEncryption.class"));	
		classFiles.add( new File(pathToCryptoClasses + "HashStuff.class"));
		classFiles.add( new File(pathToCryptoClasses + "CipherStuff.class"));
		classFiles.add( new File(pathToCryptoClasses + "SessionKeyCrypt.class"));
		
		classFiles.add( new File(pathToCryptoClasses + "kdf" + sep + "KeyDerivation.class"));
		
		classFiles.add( new File(pathToToolsClasses + "ReadResources.class"));	
		classFiles.add( new File(pathToToolsClasses + "WriteResources.class"));	
		classFiles.add( new File(pathToToolsClasses + "FileTools.class"));	
		classFiles.add( new File(pathToToolsClasses + "Converter.class"));
		classFiles.add( new File(pathToToolsClasses + "Zeroizer.class"));
		classFiles.add( new File(pathToToolsClasses + "Comparator.class"));
		classFiles.add( new File(pathToToolsClasses + "MouseRandomCollector.class"));
		classFiles.add( new File(pathToToolsClasses + "KeyRandomCollector.class"));
		classFiles.add( new File(pathToToolsClasses + "PasswordQualityCheck.class"));
		classFiles.add( new File(pathToToolsClasses + "EntropyPool.class"));
		classFiles.add( new File(pathToToolsClasses + "EntropyPool$EntropyThread.class")); // inner class
		classFiles.add( new File(pathToToolsClasses + "UnexpectedValueException.class")); // inner class
		classFiles.add( new File(pathToToolsClasses + "ZipStuff.class"));
//		classFiles.add( new File(pathToToolsClasses + "Help.class")); // inner class
		
		
		//===========================
		// bouncycastle:
		classFiles.add( new File(pathToBcClasses + "util/Arrays.class"));
		classFiles.add( new File(pathToBcClasses + "util/Memoable.class"));
		classFiles.add( new File(pathToBcClasses + "util/Pack.class"));
		classFiles.add( new File(pathToBcClasses + "util/Integers.class"));
		
		classFiles.add( new File(pathToBcClasses + "crypto/InvalidCipherTextException.class"));
		classFiles.add( new File(pathToBcClasses + "crypto/DataLengthException.class"));
		classFiles.add( new File(pathToBcClasses + "crypto/OutputLengthException.class"));
		classFiles.add( new File(pathToBcClasses + "crypto/RuntimeCryptoException.class"));
		classFiles.add( new File(pathToBcClasses + "crypto/CryptoException.class"));
		
		classFiles.add( new File(pathToBcClasses + "crypto/BlockCipher.class"));
		classFiles.add( new File(pathToBcClasses + "crypto/BufferedBlockCipher.class"));
		classFiles.add( new File(pathToBcClasses + "crypto/CipherParameters.class"));
		classFiles.add( new File(pathToBcClasses + "crypto/Digest.class"));
		classFiles.add( new File(pathToBcClasses + "crypto/ExtendedDigest.class"));
		classFiles.add( new File(pathToBcClasses + "crypto/Mac.class"));		
		classFiles.add( new File(pathToBcClasses + "crypto/DerivationParameters.class"));
		classFiles.add( new File(pathToBcClasses + "crypto/DerivationFunction.class"));
		
		classFiles.add( new File(pathToBcClasses + "crypto/SkippingStreamCipher.class"));
		classFiles.add( new File(pathToBcClasses + "crypto/StreamCipher.class"));
		classFiles.add( new File(pathToBcClasses + "crypto/StreamBlockCipher.class"));	
		classFiles.add( new File(pathToBcClasses + "crypto/SkippingCipher.class"));
		
		classFiles.add( new File(pathToBcClasses + "crypto/macs/CMac.class"));
		classFiles.add( new File(pathToBcClasses + "crypto/macs/HMac.class"));
		
		classFiles.add( new File(pathToBcClasses + "crypto/paddings/BlockCipherPadding.class"));
		classFiles.add( new File(pathToBcClasses + "crypto/paddings/PaddedBufferedBlockCipher.class"));
		classFiles.add( new File(pathToBcClasses + "crypto/paddings/PKCS7Padding.class"));
		classFiles.add( new File(pathToBcClasses + "crypto/paddings/ISO7816d4Padding.class"));
		
		classFiles.add( new File(pathToBcClasses + "crypto/modes/CBCBlockCipher.class"));
		classFiles.add( new File(pathToBcClasses + "crypto/modes/EAXBlockCipher.class"));
		classFiles.add( new File(pathToBcClasses + "crypto/modes/AEADBlockCipher.class"));
		classFiles.add( new File(pathToBcClasses + "crypto/modes/SICBlockCipher.class"));

		classFiles.add( new File(pathToBcClasses + "crypto/params/KeyParameter.class"));
		classFiles.add( new File(pathToBcClasses + "crypto/params/ParametersWithIV.class"));
		classFiles.add( new File(pathToBcClasses + "crypto/params/ParametersWithRandom.class"));
		classFiles.add( new File(pathToBcClasses + "crypto/params/AEADParameters.class"));
		classFiles.add( new File(pathToBcClasses + "crypto/params/HKDFParameters.class"));
		
		classFiles.add( new File(pathToBcClasses + "crypto/generators/HKDFBytesGenerator.class"));

		//=======================================
		// Variable classes: 
		//=======================================
		
		//===========================
		// Cryptography setting specific classes:
		//if (KDFScheme.getKDFScheme() instanceof SCryptConnector) {
		if (KeyDerivation.getKdf() instanceof ScryptKDF) {
			
			classFiles.add( new File(pathToCryptoClasses + "kdf" + sep + "ScryptKDF.class"));

			classFiles.add( new File(pathToBcClasses + "crypto/generators/SCrypt.class"));			
			classFiles.add( new File(pathToBcClasses + "crypto/PBEParametersGenerator.class"));
			classFiles.add( new File(pathToBcClasses + "crypto/generators/PKCS5S2ParametersGenerator.class"));					
			classFiles.add( new File(pathToBcClasses + "crypto/digests/SHA1Digest.class"));		
			classFiles.add( new File(pathToBcClasses + "crypto/digests/SHA256Digest.class"));
			classFiles.add( new File(pathToBcClasses + "crypto/digests/GeneralDigest.class"));
			classFiles.add( new File(pathToBcClasses + "crypto/digests/EncodableDigest.class"));
			classFiles.add( new File(pathToBcClasses + "crypto/engines/Salsa20Engine.class"));
			classFiles.add( new File(pathToBcClasses + "crypto/MaxBytesExceededException.class"));			
			classFiles.add( new File(pathToBcClasses + "util/Strings.class"));		
			
		} else if (KeyDerivation.getKdf() instanceof BcryptKDF) {
			classFiles.add( new File(pathToBcClasses + "crypto/generators/BcryptCore.class"));
			classFiles.add( new File(pathToCryptoClasses + "kdf" + sep + "BcryptKDF.class"));
		} else if (KeyDerivation.getKdf() instanceof PomeloKDF) {
			classFiles.add( new File(pathToCryptoClasses + "kdf" + sep + "PomeloKDF.class"));
			classFiles.add( new File(pathToEckCryptoClasses + "PasswordHashingScheme.class"));
		} else if (KeyDerivation.getKdf() instanceof CatenaKDF) {

			classFiles.add( new File(pathToCryptoClasses + "kdf" + sep + "CatenaKDF.class"));
			// CatenaKDF selects between all instances, so to avoid NoClassDefFoundError
			// all classes are included
			classFiles.add( new File(pathToEckCryptoClasses + "Catena.class"));
			classFiles.add( new File(pathToEckCryptoClasses + "CatenaBRG.class"));
			classFiles.add( new File(pathToEckCryptoClasses + "CatenaDBG.class"));
			classFiles.add( new File(pathToEckCryptoClasses + "CatenaHelper.class"));
			classFiles.add( new File(pathToEckCryptoClasses + "PasswordHashingScheme.class"));
			if (! (HashStuff.getHashAlgo() instanceof Blake2bDigest)) {
				classFiles.add( new File(pathToBcClasses + "crypto/digests/Blake2bDigest.class"));
			}
			classFiles.add( new File(pathToEckCryptoClasses + "Blake2b_1.class"));				
			classFiles.add( new File(pathToEckCryptoClasses + "FastHash.class"));
		} else {
			System.err.println("CreateJarFile: Invalid kdf.");
			System.exit(3);
		}
		
		if (CipherStuff.getCipherAlgo() instanceof Shacal2Engine) {
			classFiles.add( new File(pathToBcClasses + "crypto/engines/Shacal2Engine.class"));
		} else if (CipherStuff.getCipherAlgo() instanceof TwofishEngine) {
			classFiles.add( new File(pathToBcClasses + "crypto/engines/TwofishEngine.class"));
		} else if (CipherStuff.getCipherAlgo() instanceof AESEngine) {
			classFiles.add( new File(pathToBcClasses + "crypto/engines/AESEngine.class"));
		} else if (CipherStuff.getCipherAlgo() instanceof AESFastEngine) {
			classFiles.add( new File(pathToBcClasses + "crypto/engines/AESFastEngine.class"));
		} else if (CipherStuff.getCipherAlgo() instanceof ThreefishEngine) {
			
			classFiles.add( new File(pathToBcClasses + "crypto/engines/ThreefishEngine.class"));
			classFiles.add( new File(pathToBcClasses + "crypto/params/TweakableBlockCipherParameters.class"));
			classFiles.add( new File(pathToBcClasses + "crypto/engines/ThreefishEngine$ThreefishCipher.class"));
			classFiles.add( new File(pathToBcClasses + "crypto/engines/ThreefishEngine$Threefish1024Cipher.class"));
			classFiles.add( new File(pathToBcClasses + "crypto/engines/ThreefishEngine$Threefish512Cipher.class"));
			classFiles.add( new File(pathToBcClasses + "crypto/engines/ThreefishEngine$Threefish256Cipher.class"));
		} else if (CipherStuff.getCipherAlgo() instanceof SerpentEngine) {
			classFiles.add( new File(pathToBcClasses + "crypto/engines/SerpentEngine.class"));
		} else {
			System.err.println("CreateJarFile: Invalid cipher algorithm.");
			System.exit(3);			
		}
		
		if (CipherStuff.getCipherMode() instanceof EAXMode) {
			classFiles.add( new File(pathToCryptoClasses + "EAXMode.class"));	
		}
		
		if (HashStuff.getHashAlgo() instanceof WhirlpoolDigest) {
			classFiles.add(  new File(pathToBcClasses + "crypto/digests/WhirlpoolDigest.class"));
		} else if (HashStuff.getHashAlgo() instanceof SkeinDigest) {
			if ( ! (CipherStuff.getCipherAlgo() instanceof ThreefishEngine) ) {
				classFiles.add( new File(pathToBcClasses + "crypto/engines/ThreefishEngine.class"));
				classFiles.add( new File(pathToBcClasses + "crypto/engines/ThreefishEngine$ThreefishCipher.class"));
				classFiles.add( new File(pathToBcClasses + "crypto/engines/ThreefishEngine$Threefish256Cipher.class"));
				classFiles.add( new File(pathToBcClasses + "crypto/engines/ThreefishEngine$Threefish512Cipher.class"));
				classFiles.add( new File(pathToBcClasses + "crypto/engines/ThreefishEngine$Threefish1024Cipher.class"));
			}

			classFiles.add( new File(pathToBcClasses + "crypto/digests/SkeinDigest.class"));
			classFiles.add( new File(pathToBcClasses + "crypto/digests/SkeinEngine.class"));
			classFiles.add( new File(pathToBcClasses + "crypto/digests/SkeinEngine$UBI.class"));
			classFiles.add( new File(pathToBcClasses + "crypto/digests/SkeinEngine$UbiTweak.class"));
			classFiles.add( new File(pathToBcClasses + "crypto/digests/SkeinEngine$Configuration.class"));
			classFiles.add( new File(pathToBcClasses + "crypto/digests/SkeinEngine$Parameter.class"));
			
		} else if (HashStuff.getHashAlgo() instanceof SHA512Digest) {
			classFiles.add( new File(pathToBcClasses + "crypto/digests/SHA512Digest.class"));
			classFiles.add( new File(pathToBcClasses + "crypto/digests/LongDigest.class"));
			if (! (KeyDerivation.getKdf() instanceof ScryptKDF) ) {
				classFiles.add( new File(pathToBcClasses + "crypto/digests/EncodableDigest.class"));
			}
		} else if (HashStuff.getHashAlgo() instanceof SHA384Digest) {
			classFiles.add( new File(pathToBcClasses + "crypto/digests/SHA384Digest.class"));
			classFiles.add( new File(pathToBcClasses + "crypto/digests/LongDigest.class"));
			if (! (KeyDerivation.getKdf() instanceof ScryptKDF) ) {
				classFiles.add( new File(pathToBcClasses + "crypto/digests/EncodableDigest.class"));
			}
		} else if (HashStuff.getHashAlgo() instanceof SHA3Digest) {
			classFiles.add( new File(pathToBcClasses + "crypto/digests/SHA3Digest.class"));	
		} else if (HashStuff.getHashAlgo() instanceof Blake2bDigest) {
			classFiles.add( new File(pathToBcClasses + "crypto/digests/Blake2bDigest.class"));			
/*		} else if (HashStuff.getHashAlgo() instanceof RIPEMD256Digest) {
			tmpClassFiles[index++] = new File(pathToBcClasses + "crypto/digests/RIPEMD256Digest.class");
			if (! (KeyDerivation.getKdf() instanceof ScryptKDF) ) {
				tmpClassFiles[index++] = new File(pathToBcClasses + "crypto/digests/GeneralDigest.class");
			} */
		} else if (HashStuff.getHashAlgo() instanceof RIPEMD320Digest) {
			classFiles.add( new File(pathToBcClasses + "crypto/digests/RIPEMD320Digest.class"));
			if (! (KeyDerivation.getKdf() instanceof ScryptKDF) ) {
				classFiles.add( new File(pathToBcClasses + "crypto/digests/GeneralDigest.class"));
			}
		} else {
			System.err.println("CreateJarFile: Invalid hash algorithm.");
			System.exit(3);						
		}
		
		
		//===========================
		// pea type specific classes: 
		
		if (dataType instanceof FileType) {
			classFiles.add( new File(pathToEckClasses + "file_pea" + sep + "FileControl.class")); 
			classFiles.add( new File(pathToEckClasses + "file_pea" + sep + "FileControl$ShutdownHookThread.class")); // inner class
			classFiles.add( new File(pathToEckClasses + "file_pea" + sep + "LockFrameFile.class"));
			classFiles.add( new File(pathToEckClasses + "file_pea" + sep + "LockFrameFile$AccessibleLockFrameFile.class"));

		} else if (dataType instanceof ImageType) {
			classFiles.add( new File(pathToEckClasses + "image_pea" + sep + "ImageControl.class"));
			classFiles.add( new File(pathToEckClasses + "image_pea" + sep + "LockFrameImage.class"));
			classFiles.add( new File(pathToEckClasses + "image_pea" + sep + "LockFrameImage$AccessibleLockFrameImage.class"));
			
			classFiles.add( new File(pathToEckClasses + "image_pea" + sep + "ImageSetting.class"));
			classFiles.add( new File(pathToEckClasses + "image_pea" + sep + "ImageSetting$AccessibleImageSetting.class"));
			classFiles.add( new File(pathToEckClasses + "image_pea" + sep + "ImageTypePanel.class"));
			classFiles.add( new File(pathToEckClasses + "image_pea" + sep + "ImageTypePanel$AccessibleImageTypePanel.class"));

			classFiles.add( new File(pathToAllPeaClasses + "gui/SingleFileSelectionDialog.class"));
			classFiles.add( new File(pathToAllPeaClasses + "gui/SingleFileSelectionDialog$AccessibleSingleFileSelectionDialog.class"));

		} else if (dataType instanceof EditorType) {
			classFiles.add( new File(pathToEckClasses + "note_pea" + sep + "EditorControl.class"));
			classFiles.add( new File(pathToEckClasses + "note_pea" + sep + "LockFrameEditorMenu.class"));
			classFiles.add( new File(pathToEckClasses + "note_pea" + sep + "LockFrameEditorMenu$AccessibleLockFrameEditorMenu.class")); 
			classFiles.add( new File(pathToEckClasses + "note_pea" + sep + "LockFrameEditorMenu$TxtFilter.class")); // inner class
			classFiles.add( new File(pathToEckClasses + "note_pea" + sep + "LockFrameEditorMenu$RtfFilter.class")); // inner class
			classFiles.add( new File(pathToEckClasses + "note_pea" + sep + "LockFrameEditor.class"));
			classFiles.add( new File(pathToEckClasses + "note_pea" + sep + "LockFrameEditor$AccessibleLockFrameEditor.class"));
			classFiles.add( new File(pathToEckClasses + "note_pea" + sep + "CopyCutPastePopup.class"));
			classFiles.add( new File(pathToEckClasses + "note_pea" + sep + "CopyCutPastePopup$AccessibleCopyCutPastePopup.class")); 
			classFiles.add( new File(pathToEckClasses + "note_pea" + sep + "LockFrameEditor$UndoAction.class")); // inner class
			classFiles.add( new File(pathToEckClasses + "note_pea" + sep + "LockFrameEditor$RedoAction.class")); // inner class
			classFiles.add( new File(pathToEckClasses + "note_pea" + sep + "TabComponent.class"));
			classFiles.add( new File(pathToEckClasses + "note_pea" + sep + "TabComponent$AccessibleTabComponent.class"));
			
			if (FileModifier.isPswGenerator() == true) {
				classFiles.add( new File(pathToAllPeaClasses + "gui/PasswordGeneratorDialog.class"));
				classFiles.add( new File(pathToAllPeaClasses + "gui/PasswordGeneratorDialog$AccessiblePasswordGeneratorDialog.class")); 
			}	
		} else {
			System.err.println("CreateJarFile: Invalid file type.");
			System.exit(3);
		}

		//
		// stream to write jar file:
		//
		JarOutputStream jos = null;
		try {
			// create jar file with manifest			
			jos = new JarOutputStream(new FileOutputStream(jarFile), manifest);
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		//
		// Add class files to jar file
		//
		for ( File source : classFiles ) {		
			BufferedInputStream in = null;
			try  {
				// cut "bin/" and replace File.sep with "/"
				JarEntry entry = null;
				if(source.getPath().startsWith("settings")
						|| source.getPath().startsWith("start")){
					entry = new JarEntry( (source.getPath().replace(File.separator, "/") ));	
				} else {
					entry = new JarEntry( (source.getPath().substring(4, source.getPath().length())).replace(File.separator, "/") );	
				}
				entry.setTime(source.lastModified());
				jos.putNextEntry(entry);
				in = new BufferedInputStream(new FileInputStream(source));
			    byte[] buffer = new byte[1024];
			    while (true)  {
			    	int count = in.read(buffer);
			    	if (count == -1)  break;
			    	jos.write(buffer, 0, count);
			    }
			    jos.closeEntry();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}  finally {
				if (in != null) {
					try {
						in.close();
					} catch (IOException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				}
			}
		} // End for classFiles
				

		//
		// Add resource files to jar file
		//
		// from directory "resources"
		//
		File resourcesDir = new File("resources");

		if (! resourcesDir.exists() ) System.err.println("Error: resources does not exist: " + resourcesDir.getAbsolutePath() );
		if (! resourcesDir.isDirectory() ) System.err.println("Error: resources is not directory");
		if ( resourcesDir.listFiles().length == 0 ) System.err.println("Error: resources is empty");
		
		for ( File source : resourcesDir.listFiles() ) {		
			
			if (source.getName().equals("pea-lock.png") // image icon
					|| source.getName().equals("wait_cursor.png") // image icon
					|| source.getName().equals("back24.png") // image icon
					|| source.getName().equals("groesser.png") // image icon
					|| source.getName().equals("kleiner.png") // image icon
					|| source.getName().equals("weiter.png") // image icon
					|| source.getName().equals("zurueck.png") // image icon
					|| source.getName().equals("update.png") // image icon
					|| source.getName().equals("locked.png") // image icon
					|| source.getName().equals("unlocked.png") // image icon
					|| source.getName().startsWith("PeaLanguagesBundle") // language support
					// only for internal (copied) images:
					|| ( (dataType instanceof ImageType) 
							&& (source.getName().equals("text.lock"))	// encrypted image
							&& (MainView.getOpenedFileName() == null) 
							&& (PeaFactory.isBlankPea() == false)) 
					) {				
				BufferedInputStream in = null;
				try  {
					JarEntry entry = new JarEntry(source.getPath().replace(File.separator, "/"));

					entry.setTime(source.lastModified());
					jos.putNextEntry(entry);
					in = new BufferedInputStream(new FileInputStream(source));
				    byte[] buffer = new byte[1024];
				    while (true)  {
				    	int count = in.read(buffer);
				    	if (count == -1)  break;
				    	jos.write(buffer, 0, count);
				    }
				    jos.closeEntry();
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}  finally {
					if (in != null) {
						try {
							in.close();
						} catch (IOException e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						}
					}
				}
			} else {
				
			}
		} // End for resources
		
		if (jos != null) {
			try {
				jos.flush();
				jos.close();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		} 
		// set jar file executable if possible: 
		// (does not overwrite settings from partition)
		if ( ! jarFile.setExecutable(true,true) ) { // only executable for owner: second true
			try{
				Runtime.getRuntime().exec("chmod u+x " + JarStuff.getJarFileName());
			} catch (Exception e) {			
				System.out.println("Can not set jar-archive executable.");
			}
		}
			
		if (PeaFactory.isUnixScript() == true) {
			
			// jar file without ".jar"
			String jarName = JarStuff.getJarFileName().substring(0, JarStuff.getJarFileName().length() - 4);

			try {
				// create script content:
				String scriptContent = "#!/bin/sh \n"
						+ "java -jar " + JarStuff.getJarFileName();
				
				// get file name of script
				String fileName = null;
				if (MainView.getOpenedFileName() == null) { // jar file in folder
					fileName = "peas" + File.separator + jarName + File.separator 
						+ "start_" + jarName + ".sh";
				} else {
					fileName = "peas" + File.separator 
						+ "start_" + jarName + ".sh";				
				}
				
				// Write script file:
				WriteResources.writeText(scriptContent, fileName);
				
				// make script file executable:
				Runtime.getRuntime().exec("chmod u+x " + fileName);
				
			} catch (Exception e) {
				System.out.println("Error: can not create unix script");
			}			
			
		} else {
			//System.out.println("jar-archive is executable for owner.");
		}		
	}
}
