package cologne.eck.peafactory.control;

/*
 * Peafactory - Production of Password Encryption Archives
 * Copyright (C) 2015  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */

/**
 * Program flow of generation the jar file (Password Encrypted Archive). 
 */

import java.io.File;

import cologne.eck.peafactory.gui.MainView;
import cologne.eck.tools.Zeroizer;




public class JarStuff {	

	private static String jarFileName;// set in MainView, get in FileModifier
	private static String labelText;// set in MainView, get in FileModifier

	private static final String fileToCompile1 = "settings" + File.separator + "PeaSettings.java";// file to be modified: 
	private static final String fileToCompile2 = "start" + File.separator + "Start.java";// file to be modified: 


	public final void generateJarFile (byte[] keyMaterial) {
		
		// Type of PEA: image, text, file...
		DataType dataType = DataType.getCurrentType();		
		
		//=======================
		// generate java files:
		FileModifier.generateStart();		// class to start the PEA with required memory
		FileModifier.generatePeaSettings();	// all settings and Getter/Setter for the PEA

		MainView.progressBar.setValue(45);
		MainView.progressBar.paint(MainView.progressBar.getGraphics());
		
		//=====================================
		// compile with new variables
		//			
		// java files to compile
		String javaFileNames[] = {fileToCompile1, fileToCompile2};	
		FileCompiler compiler = new FileCompiler();
		compiler.compile(javaFileNames);
		MainView.progressBar.setValue(60);
		MainView.progressBar.paint(MainView.progressBar.getGraphics());		
		
		//=======================================
        // Create jar archive with compiled files
        //
		
		if ( PeaFactory.isBlankPea() == true ){ //||

			if (dataType instanceof ImageType && dataType.getData() == null) {
				//System.out.println("internal image");
				dataType.processData(keyMaterial);
				dataType.setData( "     ".getBytes() );// overwrite later
			} else {
				String dirName = JarStuff.getJarFileName().substring(0, JarStuff.getJarFileName().length() - 4); // extract ".jar"
				dirName = "peas" + File.separator + dirName;
				File dir = new File(dirName);// directory containing jar archive and resources
				if (dir.exists()){
					System.out.println("Warning: directory exists");
					// delete all files:
					DataType.clearDirectory(dir);
				} else {
					if ( !dir.mkdir() ) { // try to create
						System.err.println("JarStuff: Can not create directory: " + dirName);
						return;
					}
				}			
			}
			CreateJarFile.createJarFile();
			
		} else { // not blank PEA			
			
			if (MainView.getOpenedFileName() == null 
					&& (dataType instanceof EditorType
					|| dataType instanceof NotesType) ){ //default location for Editor and Notes
				
				// create directory to store the jar file later
				String dirName = JarStuff.getJarFileName().substring(0, JarStuff.getJarFileName().length() - 4); // extract ".jar"
				dirName = "peas" + File.separator + dirName;
				File dir = new File(dirName);// directory containing jar archive and resources
				if (dir.exists()){
					System.out.println("Warning: directory exists");
					// delete all files:
					DataType.clearDirectory(dir);
				} else {
					if ( !dir.mkdir() ) { // try to create
						System.err.println("JarStuff: Can not create directory: " + dirName);
						return;
					}
				}						
			}
		
			// Handle internal image (image inside the jar archive):
			// This can be included in the archive, because there is 
			// no need to modify the content
			if ( dataType instanceof ImageType 				// image
					&& MainView.getOpenedFileName() == null){ // no external file selected
			
				// first process data and create the resource file text.lock
				dataType.processData(keyMaterial);
				// then create jar file and add resource file
				CreateJarFile.createJarFile();// includes text.lock in jar if changeable false

			} else {
				// first create jar file
				CreateJarFile.createJarFile();// includes text.lock in jar if changeable false
				// then process data and store eventually beside the previously created jar file
				dataType.processData(keyMaterial);
			}
		}
		
		Zeroizer.zero(keyMaterial);		
		
		// overwrite random values: overwrite files with 0
		Zeroizer.zeroFile("settings" + File.separator + "PeaSettings.class");
		Zeroizer.zeroFile("settings" + File.separator + "PeaSettings.java");

		MainView.progressBar.setValue(100);
		MainView.progressBar.paint(MainView.progressBar.getGraphics());
	}
	
	
	public final static String getJarFileName() {
		return jarFileName;
	}
	public final static void setJarFileName(String _jarFileName) {
		jarFileName = _jarFileName;
	}
	public final static String getLabelText() {
		return labelText;
	}
	public final static void setLabelText(String _labelText) {
		labelText = _labelText;
	}
	public final static String getFileToCompile1(){
		return fileToCompile1;
	}
	public final static String getFileToCompile2(){
		return fileToCompile2;
	}
}
