package cologne.eck.peafactory.control;

/*
 * Peafactory - Production of Password Encryption Archives
 * Copyright (C) 2015  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */

/**
 * Generation of the note pea. 
 */

import java.io.File;


import cologne.eck.peafactory.crypto.CipherStuff;
import cologne.eck.peafactory.gui.MainView;
import cologne.eck.peafactory.gui.TextTypePanel;
import cologne.eck.tools.Converter;
import cologne.eck.tools.ReadResources;
import cologne.eck.tools.WriteResources;
import cologne.eck.tools.Zeroizer;


public final class NotesType extends DataType {

	private static byte[] textBytes = null;

	@Override
	public void processData(byte[] keyMaterial) {
		
		if (PeaFactory.isBlankPea() == true){
			// create directory to contain jar file and start files
			String dirName = JarStuff.getJarFileName().substring(0, (JarStuff.getJarFileName().length() - 4)); // extract ".jar"
			new File(dirName).mkdirs();
		} else { // not blank PEA				
			
			byte[] plainBytes = null;
			byte[] cipherBytes = null;
			
			if (MainView.getOpenedFileName() != null) {
				plainBytes = ReadResources.readExternFile(MainView.getOpenedFileName());
				
				cipherBytes = CipherStuff.getInstance().encrypt(plainBytes, keyMaterial, false);	
				
				// overwrite external file with cipherBytes:
				WriteResources.write(cipherBytes,  MainView.getOpenedFileName() , null);
			} else { // default file in directory resources
				char[] text = TextTypePanel.getText();
				TextTypePanel.resetText();
				plainBytes = Converter.chars2bytes(text);
				if (plainBytes.length > 0){				
					Zeroizer.zero(text);
				}
				
				cipherBytes = CipherStuff.getInstance().encrypt(plainBytes, keyMaterial, false);	
				
				// get name of directory containing jar archive and resources
				String dirName = JarStuff.getJarFileName().substring(0, JarStuff.getJarFileName().length() - 4); // extract ".jar"

				// store cipherBytes beside jar archive:
				WriteResources.write(cipherBytes, "text.lock", "peas" + File.separator + dirName + File.separator + "resources");
				Zeroizer.zero(cipherBytes);
			}
		}
	}

	@Override
	public String getDescription() {
		return PeaFactory.getLanguagesBundle().getString("notes_description");
	}

	@Override
	public String getTypeName() {
		return "notes";
	}

	@Override
	public byte[] getData() {
		return textBytes;
	}

	@Override
	public void setData(byte[] data) {
		textBytes = data;		
	}
}