package cologne.eck.peafactory.control;

/*
 * PeaFactory - Production of Password Encryption Archives
 * Copyright (C) 2015  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */



import java.awt.EventQueue;
import java.awt.Point;
import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.*;

import javax.swing.JOptionPane;

import cologne.eck.all_peas.control.PeaControl;
import cologne.eck.all_peas.data.Attachments;
import cologne.eck.all_peas.data.PeaProperties;
import cologne.eck.all_peas.files.FileComposer;
import cologne.eck.all_peas.files.FileModel;
import cologne.eck.all_peas.files.FileTypePanel;
import cologne.eck.all_peas.gui.PswDialogView;
import cologne.eck.peafactory.crypto.CipherStuff;
import cologne.eck.peafactory.crypto.HashStuff;
import cologne.eck.peafactory.crypto.RandomStuff;
import cologne.eck.peafactory.crypto.kdf.CatenaKDF;
import cologne.eck.peafactory.crypto.kdf.KeyDerivation;
import cologne.eck.peafactory.gui.MainView;
import cologne.eck.peafactory.gui.ProjectSelection;
import cologne.eck.tools.Zeroizer;
import cologne.eck.tools.Comparator;

import org.bouncycastle.crypto.BlockCipher;
import org.bouncycastle.crypto.Digest;
import org.bouncycastle.crypto.digests.Blake2bDigest;
import org.bouncycastle.crypto.engines.ThreefishEngine;

public final class PeaFactory {

	// i18n: 	
	private static String language; 
	private static Locale currentLocale;
	private static Locale defaultLocale = Locale.getDefault();
	private static ResourceBundle languageBundle;
	
	private static String os = null;

	private static MainView frame;
	
	private static String version = "Peafactory version 0.2.4-alpha";
	
	private final static String packagePathName = "cologne" + File.separator + "eck" + File.separator + "peafactory";
	
	// default algos:
	private static final KeyDerivation DEFAULT_KDF = new CatenaKDF();
	private static final Digest DEFAULT_HASH = new Blake2bDigest();
	private static final BlockCipher DEFAULT_CIPHER = new ThreefishEngine(512);
	
	/*
	 * Properties of the producing PEA:
	 */
	// create new random bytes for programRandomBytes and fileIdentifier
	private static boolean keyFileProperty = true;
	// create new random bytes for programRandomBytes and fileIdentifier
	// and store them to use the same values for the next PEAs
	private static boolean keepRandomBytes = false;
	// PEA with password and content or without
	private static boolean blankPea = false; // if true: create a blank, uninitialized default PEA without key file property
	// create a start script for Unix and store along with the PEA 
	private static boolean unixScript = false;
	
	// FileComposer:
	private static FileComposer fc;
	
	// Use the previously stored random bytes for programRandomBytes and fileIdentifier
	private static boolean useKeptRandomBytes = false;
	private static byte[] keptRandomBytes = null;
	private static byte[] keptFileIdentifier = null;
	
	private static boolean testMode = false;
	
	// 129 byte string to initialize the salt: 
	private final static String FIXED_SALT_STRING = "PEAFACTORY - PRODUCTION OF PASSWORD ENCRYPTED ARCHIVES - FIXED STRING USED TO INITIALIZE THE SALT FOR THE KEY DERIVATION FUNCTION";
	private final static String FIXED_FILE_ID_STRING = "PFACTORY";

	
	public static void main(String[] args) throws MalformedURLException {		
		
		os = System.getProperty("os.name");
		
		if (args.length > 0) {
			if (args[0] != null) {
				if(args[0].equals("-r")){
					System.out.println("Rescue mode only works for PEAs, not for PeaFactory...");
				} else if(args[0].equals("-t")){
					System.out.println("#####---TEST MODE---#####");
					testMode = true;
				} else {
					System.out.println("Unknown argument: " + args[0]);
				}
			}
		}
		if (testMode == true) {
			
			new TestMode().runInTestMode();
		}
		
		// i18n: 
	    language = defaultLocale.getLanguage(); // get default language
	    currentLocale = new Locale(language); // set default language as language

	    //File file = new File("src" + File.separator + "config" + File.separator + "i18n");
	    File file = new File("config" + File.separator + "i18n"); 
	    URL[] urls = {file.toURI().toURL()};  
	    ClassLoader loader = new URLClassLoader(urls);  
	    languageBundle = ResourceBundle.getBundle("LanguagesBundle", currentLocale, loader);  
    
	    // DEFAULT HASH FUNCTION: 
	    if (HashStuff.getHashAlgo() == null) { // needed for RandomPasswordDialog
	    	HashStuff.setHashAlgo( DEFAULT_HASH );
	    	HashStuff.setHMacAlgo( DEFAULT_HASH );
	    }	    
	    
		EventQueue.invokeLater(new Runnable() {
			public void run() {
				try {		
					
					PswDialogView.setUI(false);
					
					// select the kind of pea
					ProjectSelection proj = new ProjectSelection(new Point(100,10));
					//proj.setLocation( 100, 100 );
					proj.setVisible(true);				
					
					// initialize FileTypePanel:
					FileModel model= new FileModel();
					fc = new FileComposer(model);
					FileTypePanel filePanel = new FileTypePanel(
							300, 120, true, true, fc, proj);
					//MainView.setFilePanel(filePanel);
					PeaProperties.setTypePanel(filePanel);
					
					frame = MainView.getInstance(proj.getLocationOnScreen());
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});

	}
	
	public final static void createFile() {
		
		String peaFileName;
		String peaPath;
		String fileOrDirectory;
		try {
			// indicate that program is working:
			MainView.showWaitCursor();

		if (testMode == true){
			System.out.println("blank: " + PeaFactory.isBlankPea());
			System.out.println("key file property: " + PeaFactory.isKeyFileProperty());
			System.out.println("should keep random bytes for next PEA: " + keepRandomBytes);
			System.out.println("is compatible with previous PEA: " + PeaFactory.isUseKeptRandomBytes());
		}

		
		//=====================
		// initialize (order!)
		//
	    if (HashStuff.getHashAlgo() == null) {
	    	HashStuff.setHashAlgo( DEFAULT_HASH );// default
	    	HashStuff.setHMacAlgo( DEFAULT_HASH );// default
	    }
	    if (PeaFactory.isKeyFileProperty() == true){ // unique values
	    	
	    	// generate random values for fileIdentifier and Salt
	    	if (keepRandomBytes == true) { // keep values for next PEA
	    		
	    		if (useKeptRandomBytes == true){ // use values from previous PEA
	    			// check for program bug in test mode:
	    			if (testMode == true) {
	    				Comparator.checkNullVector(keptRandomBytes);
	    			}
	    			Attachments.setProgramRandomBytes(keptRandomBytes);
	    		} else { // create new values
	    			Attachments.generateAndSetProgramRandomBytes();
	    		}
	    		// store values in variable:
	    		keptRandomBytes = Attachments.getProgramRandomBytes();
	    		
	    		if (useKeptRandomBytes == true){ // use values from previous PEA
	    			Attachments.setFileIdentifier(keptFileIdentifier);
	    		} else { // create new values
	    			Attachments.generateAndSetFileIdentifier();	
	    		}		    	
	    		// store values in variable:
		    	keptFileIdentifier = Attachments.getFileIdentifier();
		    	// set value to register in next PEA
		    	useKeptRandomBytes = true;
		    	
	    	} else {
	    		if (keptRandomBytes != null) {
	    			Zeroizer.zero(keptRandomBytes);
	    			keptRandomBytes = null;
	    			keptFileIdentifier = null;
	    		}
		    	Attachments.generateAndSetProgramRandomBytes();
		    	Attachments.generateAndSetFileIdentifier();	
		    	useKeptRandomBytes = false;
	    	}

	    } else { // default values
	    	
	    	useKeptRandomBytes = false;
    		if (keptRandomBytes != null) {
    			Zeroizer.zero(keptRandomBytes);
    			keptRandomBytes = null;
    		}
	    	// use default values for fileIdentifier and programRandomBytes
	    	// calculate default salt and update via setAttachedSalt: 
	    	// xor the salt with attachedSalt (will be attached to the content)
	    	Attachments.setFileIdentifier(FIXED_FILE_ID_STRING.getBytes(PeaProperties.getCharset())); // 8 byte
	    	String saltString = FIXED_SALT_STRING.substring(0, KeyDerivation.getSaltSize() );
	    	Attachments.setProgramRandomBytes(saltString.getBytes(PeaProperties.getCharset()));
	    }	    
	    //Help.printBytes("programRandomBytes",Attachments.getProgramRandomBytes());
	    
	    // new
	    byte[] attachedSalt = new RandomStuff().createRandomBytes(Attachments.getProgramRandomBytesSize());
	    KeyDerivation.setSalt(attachedSalt);
	    
	    // DEFAULT CIPHER ALGO: 
	    if (CipherStuff.getCipherAlgo() == null) {
	    	CipherStuff.setCipherAlgo( DEFAULT_CIPHER );
	    }
	    // DEFAULT KEY DERIVATION FUNCTION
	    if (KeyDerivation.getKdf() == null) {
	    	KeyDerivation.setKdf( DEFAULT_KDF );
	    }

		//======================================
		// get password from pswField and check:
		//
		char[] pswInputChars = MainView.getPsw();	
		MainView.resetPsw();
		char[] pswInputChars2 = MainView.getPsw2();
		MainView.resetPsw2();
		
		if ( ( pswInputChars2.length == 0)  && ( pswInputChars.length != 0)) {
			JOptionPane.showMessageDialog(null, languageBundle.getString("password2_is_null"), null, JOptionPane.PLAIN_MESSAGE);
			return;
		} else if (pswInputChars.length != pswInputChars2.length){
			JOptionPane.showMessageDialog(null, languageBundle.getString("different_password"), null, JOptionPane.PLAIN_MESSAGE);
			return;
		}
		boolean pswEqual = true;
		// prevent timing attacks: 
		for (int i = 0; i < pswInputChars.length; i++) {
			if (pswInputChars[i] != pswInputChars2[i]) {
				pswEqual = false;
			}
		}		
		if (pswEqual == false) {
			JOptionPane.showMessageDialog(null, languageBundle.getString("different_password"), null, JOptionPane.PLAIN_MESSAGE);
			return;
		}

		
		int inputLength = pswInputChars.length;

		Arrays.fill(pswInputChars2, '\0');		
		// Warning for insecure password
		if ((inputLength == 0) && (PeaFactory.isBlankPea() == false)) {
			// allow null-passwords with warning
			JOptionPane.showMessageDialog(frame,
					languageBundle.getString("no_password_warning"),
				"Warning",
				JOptionPane.WARNING_MESSAGE);
			pswInputChars = "no password".toCharArray();

		} else if (inputLength < 12 && PeaFactory.isBlankPea() == false) {
			JOptionPane.showMessageDialog(null, languageBundle.getString("short_password_warning"), null, JOptionPane.PLAIN_MESSAGE);
		}

		//=============
		//  derive key: 
		//	

		byte[] keyMaterial = null;
		
		//Help.printBytes("attachedSalt", KeyDerivation.getAttachedSalt());
		//Help.printBytes("programRandomBytes", Attachments.getProgramRandomBytes());
		//Help.printBytes("salt",  KeyDerivation.getSalt());
		
		if (PeaFactory.isBlankPea() == false) {// if true: no need to derive key
			
			keyMaterial = PeaControl.deriveKeyFromPsw(pswInputChars);
			// check for program bug in test mode:
			if (testMode == true) {
				Comparator.checkNullVector(keyMaterial);
			}
			
			if (keyMaterial == null) {
				JOptionPane.showMessageDialog(frame,
						languageBundle.getString("program_bug")
						+ ": \n  (keyMaterial is null)",
					"Error",
					JOptionPane.ERROR_MESSAGE);
				return;
			}		
		}
		MainView.progressBar.setValue(35);
		MainView.progressBar.paint(MainView.progressBar.getGraphics());
			
		//=================
		// generate jar file:
		//
		//long start = System.currentTimeMillis(); // Startpunkt

		new JarStuff().generateJarFile(keyMaterial);
		
		//System.out.println("generateJarFile - Zeit in ms: \n" + (System.currentTimeMillis() - start));//Messpunkt
		
		fileOrDirectory = (MainView.getOpenedFileName() == null) ? languageBundle.getString("directory") : languageBundle.getString("file");
		if (PeaFactory.isBlankPea() == true) {
			fileOrDirectory = languageBundle.getString("directory"); // for all peas
		}
		if (DataType.getCurrentType() instanceof FileType) {
			fileOrDirectory = languageBundle.getString("file");
		}
		peaFileName = JarStuff.getJarFileName().substring(0, (JarStuff.getJarFileName().length() - 4)); // extract ".jar"
		peaPath = System.getProperty("user.dir") + File.separator + "peas";
	} finally {
		MainView.resetCursor();
	}
		
		// show informations about created pea:
		int input = JOptionPane.showConfirmDialog(frame,
				languageBundle.getString("new_pea_info") + "\n"
				+ DataType.getCurrentType().getDescription() + "\n\n"
				+ languageBundle.getString("new_pea_type_info") + " "
				+ fileOrDirectory + " \n "
				+ "    - " + peaFileName + " -\n"
				+ languageBundle.getString("new_pea_location_info") + "\n "
				+ "   " + peaPath + "\n\n"
				+ languageBundle.getString("open_directory") + "\n ",
			"Pea info",
				    JOptionPane.YES_NO_OPTION);
		
		if (input == JOptionPane.YES_OPTION) { // 0 
			
			String error = OperatingSystemStuff.openFileManager(peaPath);
			
			if (error != null) {
				JOptionPane.showMessageDialog(frame,
						languageBundle.getString("desktop_error")
						+ "\n\n  " + error,
						languageBundle.getString("error"),
						JOptionPane.ERROR_MESSAGE);
			}
		} else { // do not open folder peas:
			//
		}
	}
	
	//=========================
	// Helper functions:
	
	// set new language, locale, ResourceBundle
	public final static void setI18n(String newLanguage){
        
        // get ResourceBundle
	    File file = new File("config" + File.separator + "i18n");  
	    URL url = null;
		try {
			url = file.toURI().toURL();
		} catch (MalformedURLException e) {
			System.err.println("PeaFactory: Missing Language files.");
			// TODO Auto-generated catch block
			e.printStackTrace();
		}  
	    URL[] urls = {url};
	    ClassLoader loader = new URLClassLoader(urls);  
	    ResourceBundle newLanguageBundle = ResourceBundle.getBundle("LanguagesBundle", new Locale(newLanguage), loader);  

		// set variables:
	    language = newLanguage;
	    currentLocale = new Locale(newLanguage);
	    languageBundle = newLanguageBundle;

		MainView.updateFrame(MainView.getFrameLocation());
	}

	// ==============================================================================================
	// Getter & Setter
	//
	public final static String getLanguage() {
		return language;
	}
	public final static ResourceBundle getLanguagesBundle(){
		return languageBundle;
	}
	public final static String getPackagePath() {
		return packagePathName;
	}
	public final static boolean isFrameInstantiated() {
		if (frame == null) {
			return false;
		} else {
			return true;
		}
	}

	public final static String getOS() {
		return os;
	}
	public final static MainView getFrame() {
		return frame;
	}
	public final static KeyDerivation getDefaultKDF() {
		return DEFAULT_KDF;
	}
	public static BlockCipher getDefaultCipher() {
		return DEFAULT_CIPHER;
	}
	public static Digest getDefaultHash() {
		return DEFAULT_HASH;
	}

	/**
	 * @return the version
	 */
	public static String getVersion() {
		return version;
	}

	/**
	 * Check if the next to produce PEA will be compatible with the current PEA. 
	 * Compatible PEAs do reuse the previously stored random values. 
	 * 
	 * @return the keepRandomBytes	true, if next to produce PEA will be compatible with the current PEA. 
	 */
	public static boolean isKeepRandomBytes() {
		return keepRandomBytes;
	}

	/**
	 * Set compatibility of the next PEA, that means reusing of the random bytes 
	 * of the current PEA for the next to produce PEA. 
	 * 
	 * @param keepRandomBytes the keepRandomBytes to set
	 * 					true: the next to produce PEA will be compatible with the current PEA. 
	 * 					false: next PEA will be incompatible, random values will be cleared.
	 */
	public static void setKeepRandomBytes(boolean keepRandomBytes) {
		PeaFactory.keepRandomBytes = keepRandomBytes;
	}

	/**
	 * Check if existing random bytes should be reused. These random
	 * bytes had been stored previously for reusing. 
	 * 
	 * @return	true if random bytes should be reused
	 */
	public static boolean isUseKeptRandomBytes() {
		return useKeptRandomBytes;
	}
	/**
	 * Set reusing existing random bytes. These random
	 * bytes had been stored previously for reusing. 
	 * 
	 * @param _useKeptRandomBytes	true: reuse, otherwise clear the random bytes
	 */
	public static void setUseKeptRandomBytes(boolean _useKeptRandomBytes) {
		useKeptRandomBytes = _useKeptRandomBytes;
	}

	/**
	 * @return the blankPea
	 */
	public static boolean isBlankPea() {
		return blankPea;
	}

	/**
	 * @param blankPea the blankPea to set
	 */
	public static void setBlankPea(boolean _blankPea) {
		PeaFactory.blankPea = _blankPea;
	}

	/**
	 * @return the unixScript
	 */
	public static boolean isUnixScript() {
		return unixScript;
	}

	/**
	 * @param unixScript the unixScript to set
	 */
	public static void setUnixScript(boolean _unixScript) {
		PeaFactory.unixScript = _unixScript;
	}

	/**
	 * @return the keyFileProperty
	 */
	public static boolean isKeyFileProperty() {
		return keyFileProperty;
	}

	/**
	 * @param keyFileProperty the keyFileProperty to set
	 */
	public static void setKeyFileProperty(boolean _keyFileProperty) {
		PeaFactory.keyFileProperty = _keyFileProperty;
	}
}