package cologne.eck.peafactory.gui;

/*
 * Peafactory - Production of Password Encryption Archives
 * Copyright (C) 2015  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */

/**
 * Settings for Peas
 */


import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ResourceBundle;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextArea;

import cologne.eck.all_peas.gui.PeaBorderFactory;
import cologne.eck.peafactory.control.PeaFactory;






@SuppressWarnings("serial")
public class GeneralPeaSettings extends JDialog implements ActionListener {	
	
	private GeneralPeaSettings peaSettings;
		
	private JCheckBox keyFilePropertyCheck;
	private JCheckBox compatibilityCheck;
	private JCheckBox defaultCheck;	
	private JCheckBox blankCheck;
	private JCheckBox scriptCheck;

	
	protected ResourceBundle languageBundle;
	
	public GeneralPeaSettings() {
		
		super(PeaFactory.getFrame() );

		peaSettings = this;
// absturz...		
//		peaSettings.setAlwaysOnTop(true);		

		// i18n
		languageBundle = PeaFactory.getLanguagesBundle();
		
		this.setIconImage(MainView.getImage());
		
		this.setTitle(languageBundle.getString("general_pea_settings"));

		JPanel contentPane = (JPanel) peaSettings.getContentPane();//new JPanel();
		contentPane.setBorder(PeaBorderFactory.createBorder(false));
		contentPane.setLayout(new BoxLayout(contentPane, BoxLayout.Y_AXIS));
		
		contentPane.add(Box.createVerticalStrut(10));
		
		JPanel infoPanel = new JPanel();
		infoPanel.setLayout(new BoxLayout(infoPanel, BoxLayout.X_AXIS));
		JTextArea infoArea = new JTextArea(languageBundle.getString("general_pea_infos"));
		infoArea.setWrapStyleWord(true);
		infoArea.setLineWrap(true);
		infoArea.setEditable(false);
		infoPanel.add(Box.createHorizontalStrut(10));
		infoPanel.add(infoArea);
		infoPanel.add(Box.createHorizontalGlue());

		contentPane.add(infoPanel);
		
		contentPane.add(Box.createVerticalStrut(10));		
		
		JPanel tipPanel = new JPanel();
		tipPanel.setLayout(new BoxLayout(tipPanel, BoxLayout.X_AXIS));
		JLabel tipLabel1 = new JLabel(languageBundle.getString("this_session"));
		tipLabel1.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 11));
		tipPanel.add(tipLabel1);
		tipPanel.add(Box.createHorizontalGlue());
		
		contentPane.add(tipPanel);		
		
		contentPane.add(Box.createVerticalStrut(20));
		
		JPanel boundPanel = new JPanel();
		
		boundPanel.setLayout(new BoxLayout(boundPanel, BoxLayout.X_AXIS));
		keyFilePropertyCheck = new JCheckBox(languageBundle.getString("key_file_pea"), false);
		keyFilePropertyCheck.setSelected(true);
		keyFilePropertyCheck.addActionListener(this);
		keyFilePropertyCheck.setActionCommand("keyFilePropertyCheck");

		boundPanel.add(keyFilePropertyCheck);
		boundPanel.add(Box.createHorizontalGlue());
		
		contentPane.add(boundPanel);		
		
		JPanel infoBoundPanel = new JPanel();
		infoBoundPanel.setLayout(new BoxLayout(infoBoundPanel, BoxLayout.X_AXIS));
		JTextArea infoBoundArea = new JTextArea(languageBundle.getString("key_file_pea_infos"));
		infoBoundArea.setWrapStyleWord(true);
		infoBoundArea.setLineWrap(true);
		infoBoundArea.setEditable(false);
		
		infoBoundPanel.add(Box.createHorizontalStrut(20));
		infoBoundPanel.add(infoBoundArea);
		infoBoundPanel.add(Box.createHorizontalGlue());
		
		contentPane.add(infoBoundPanel);
		contentPane.add(Box.createVerticalStrut(10));
		
		
		JPanel defaultPanel = new JPanel();
		defaultPanel.setLayout(new BoxLayout(defaultPanel, BoxLayout.X_AXIS));
		defaultCheck = new JCheckBox(languageBundle.getString("default_pea"), false);
		defaultCheck.setSelected(false);
		defaultCheck.addActionListener(this);
		defaultCheck.setActionCommand("default");
		
		defaultPanel.add(defaultCheck);
		defaultPanel.add(Box.createHorizontalGlue());
		contentPane.add(defaultPanel);
		
		JPanel infoDefaultPanel = new JPanel();
		infoDefaultPanel.setLayout(new BoxLayout(infoDefaultPanel, BoxLayout.X_AXIS));
		JTextArea infoDefaultArea = new JTextArea(languageBundle.getString("info_default_pea"));
		infoDefaultArea.setWrapStyleWord(true);
		infoDefaultArea.setLineWrap(true);
		infoDefaultArea.setEditable(false);
		
		infoDefaultPanel.add(Box.createHorizontalStrut(20));
		infoDefaultPanel.add(infoDefaultArea);
		infoDefaultPanel.add(Box.createHorizontalGlue());

		contentPane.add(infoDefaultPanel);		
		contentPane.add(Box.createVerticalStrut(10));
		
		
		JPanel compatibilityPanel = new JPanel();
		compatibilityPanel.setLayout(new BoxLayout(compatibilityPanel, BoxLayout.X_AXIS));
		compatibilityCheck = new JCheckBox(languageBundle.getString("compatible_pea"), false);
		compatibilityCheck.setSelected(false);
		compatibilityCheck.addActionListener(this);
		compatibilityCheck.setActionCommand("nextPeaCompatible");
		
		compatibilityPanel.add(compatibilityCheck);
		compatibilityPanel.add(Box.createHorizontalGlue());
		contentPane.add(compatibilityPanel);
		
		JPanel infoCompatibilityPanel = new JPanel();
		infoCompatibilityPanel.setLayout(new BoxLayout(infoCompatibilityPanel, BoxLayout.X_AXIS));
		JTextArea infoCompatibilityArea = new JTextArea(languageBundle.getString("info_compatible_pea"));
		infoCompatibilityArea.setWrapStyleWord(true);
		infoCompatibilityArea.setLineWrap(true);
		infoCompatibilityArea.setEditable(false);
		
		infoCompatibilityPanel.add(Box.createHorizontalStrut(20));
		infoCompatibilityPanel.add(infoCompatibilityArea);
		infoCompatibilityPanel.add(Box.createHorizontalGlue());

		contentPane.add(infoCompatibilityPanel);		
		contentPane.add(Box.createVerticalStrut(10));
		
		
		JPanel blankPanel = new JPanel();
		blankPanel.setLayout(new BoxLayout(blankPanel, BoxLayout.X_AXIS));
		blankCheck = new JCheckBox(languageBundle.getString("blank_pea"), false);
		blankCheck.setSelected(false);
		blankCheck.addActionListener(this);
		blankCheck.setActionCommand("blankCheck");
		
		blankPanel.add(blankCheck);
		blankPanel.add(Box.createHorizontalGlue());
		contentPane.add(blankPanel);
		
		JPanel infoBlankPanel = new JPanel();
		infoBlankPanel.setLayout(new BoxLayout(infoBlankPanel, BoxLayout.X_AXIS));
		JTextArea infoBlankArea = new JTextArea(languageBundle.getString("blank_pea_infos"));
		infoBlankArea.setWrapStyleWord(true);
		infoBlankArea.setLineWrap(true);
		infoBlankArea.setEditable(false);
		
		infoBlankPanel.add(Box.createHorizontalStrut(20));
		infoBlankPanel.add(infoBlankArea);
		infoBlankPanel.add(Box.createHorizontalGlue());

		contentPane.add(infoBlankPanel);		
		contentPane.add(Box.createVerticalStrut(10));
		
		
		JPanel scriptPanel = new JPanel();
		scriptPanel.setLayout(new BoxLayout(scriptPanel, BoxLayout.X_AXIS));
		scriptCheck = new JCheckBox(languageBundle.getString("script_check"), false);
		scriptCheck.setSelected(false);
		scriptCheck.addActionListener(this);
		scriptCheck.setActionCommand("unixScript");
		
		scriptPanel.add(scriptCheck);
		scriptPanel.add(Box.createHorizontalGlue());
		contentPane.add(scriptPanel);
		
		JPanel infoScriptPanel = new JPanel();
		infoScriptPanel.setLayout(new BoxLayout(infoScriptPanel, BoxLayout.X_AXIS));
		JTextArea infoScriptArea = new JTextArea(languageBundle.getString("info_script_check"));
		infoScriptArea.setWrapStyleWord(true);
		infoScriptArea.setLineWrap(true);
		infoScriptArea.setEditable(false);
		
		infoScriptPanel.add(Box.createHorizontalStrut(20));
		infoScriptPanel.add(infoScriptArea);
		infoScriptPanel.add(Box.createHorizontalGlue());

		contentPane.add(infoScriptPanel);
		contentPane.add(Box.createVerticalStrut(10));
		
		JPanel buttonPanel = new JPanel();
		buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
		JButton okButton = new JButton("ok");
		okButton.setActionCommand("ok");
		okButton.addActionListener(this);
		buttonPanel.add(Box.createHorizontalGlue());
		buttonPanel.add(okButton);
		
		contentPane.add(buttonPanel);
		
		peaSettings.setSize(new Dimension(550,550));
		//peaSettings.setLocation(PeaFactory.getFrame().getLocation());
		peaSettings.setLocation(MainView.getFrameLocation());
		//peaSettings.pack();
		peaSettings.setVisible(true);
	}

	@Override
	public void actionPerformed(ActionEvent ape) {
		String com = ape.getActionCommand();
		
		if (com.equals("ok")) {

			if (blankCheck.isSelected()) {
				
				PeaFactory.setBlankPea(true);			
				PeaFactory.setKeyFileProperty(false);
				PeaFactory.setKeepRandomBytes(false);
				PeaFactory.setUseKeptRandomBytes(false);

				MainView.updateFrame(MainView.getFrameLocation());
				MainView.getInstance(MainView.getFrameLocation()).updateLabelText(languageBundle.getString("blank_pea"));
				
			} else if (defaultCheck.isSelected()) {
				
				PeaFactory.setBlankPea(false);
				PeaFactory.setKeyFileProperty(false);
				PeaFactory.setKeepRandomBytes(false);
				PeaFactory.setUseKeptRandomBytes(false);

				MainView.updateFrame(MainView.getFrameLocation());
				MainView.getInstance(MainView.getFrameLocation()).updateLabelText(languageBundle.getString("default_pea"));

			} else if (compatibilityCheck.isSelected() ) {
				
				PeaFactory.setBlankPea(false);
				PeaFactory.setKeyFileProperty(true);
				
				// PeaFactoty should keep random bytes for next PEA
				// while keeping these bytes, MainView.isCompatibleWithPreviousPEA is set in PeaFactory
				PeaFactory.setKeepRandomBytes(true);

				MainView.updateFrame(MainView.getFrameLocation());
				MainView.getInstance(MainView.getFrameLocation()).updateLabelText(languageBundle.getString("key_file_pea"));

			} else if (keyFilePropertyCheck.isSelected() ) {
				
				PeaFactory.setBlankPea(false);
				PeaFactory.setKeyFileProperty(true);
				PeaFactory.setKeepRandomBytes(false);

				MainView.updateFrame(MainView.getFrameLocation());
				MainView.getInstance(MainView.getFrameLocation()).updateLabelText(languageBundle.getString("key_file_pea"));
			}
			
			if (scriptCheck.isSelected() ) {
				PeaFactory.setUnixScript(true);
			} else {
				PeaFactory.setUnixScript(false);
			}

			peaSettings.dispose();
			 
		} else if ( com.startsWith("blankCheck")) {	
			
			if (blankCheck.isSelected()) {
				defaultCheck.setSelected(true);
				keyFilePropertyCheck.setSelected(false);
				compatibilityCheck.setSelected(false);
			} 
			
		} else if ( com.startsWith("default")) {	
			
			if (defaultCheck.isSelected()) {
				keyFilePropertyCheck.setSelected(false);
				compatibilityCheck.setSelected(false);
			} else {
				keyFilePropertyCheck.setSelected(true);
			}
			
		} else if ( com.startsWith("keyFilePropertyCheck")) {	
			
			if (keyFilePropertyCheck.isSelected()) {
				blankCheck.setSelected(false);
				defaultCheck.setSelected(false);
			} else {
				defaultCheck.setSelected(true);
			}
		
		} else if ( com.startsWith("nextPeaCompatible")) {	
		
			if (compatibilityCheck.isSelected()) {				
				keyFilePropertyCheck.setSelected(true);
				blankCheck.setSelected(false);
				defaultCheck.setSelected(false);
			} 		
		}
	}
}
