package cologne.eck.peafactory.gui;

/*
 * Peafactory - Production of Password Encryption Archives
 * Copyright (C) 2015  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */

/**
 * Main frame for the production of any Password Encryption Archive (pea).
 */

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.*;
import java.util.Arrays;
import java.util.ResourceBundle;

import javax.swing.*;
//import javax.swing.border.EmptyBorder;
import javax.swing.border.LineBorder;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import cologne.eck.all_peas.data.PeaProperties;
import cologne.eck.all_peas.files.FileTypePanel;
import cologne.eck.all_peas.gui.CharTable;
import cologne.eck.all_peas.gui.CursorManager;
import cologne.eck.all_peas.gui.PeaBorderFactory;
import cologne.eck.image_pea.ImageTypePanel;
import cologne.eck.peafactory.control.DataType;
import cologne.eck.peafactory.control.EditorType;
import cologne.eck.peafactory.control.FileType;
import cologne.eck.peafactory.control.ImageType;
import cologne.eck.peafactory.control.JarStuff;
import cologne.eck.peafactory.control.NotesType;
import cologne.eck.peafactory.control.PeaFactory;
import cologne.eck.tools.EntropyPool;
import cologne.eck.tools.KeyRandomCollector;
import cologne.eck.tools.MouseRandomCollector;
import cologne.eck.tools.PasswordQualityCheck;
import cologne.eck.tools.Zeroizer;




@SuppressWarnings("serial")
public final class MainView extends JFrame implements ActionListener {

	// static instance
	private static MainView frame; 
	
	// file to encrypt: 
	// if null, this indicates that resource is saved beside jar archive in same directory
	private static String openedFileName = null;

	// screen: 
	private static Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
	private static double screenWidth = screenSize.getWidth();
	private static double screenHeight = screenSize.getHeight();
	
	private JLabel projectLabel; // Notebook, Image...
	
	private static FileTypePanel filePanel;
	
	// MainView is several times disposed and new created, so
	// password fields must be static
	private static JPasswordField pswField;
	private static JPasswordField pswField2;
	
	private JLabel qualityLabel;
	private JLabel qualityLabelStars;

	private JTextField jarNameField;
	private JTextField labelTextField;	

	protected JLabel pathLabel;
	public static JProgressBar progressBar;
	
	protected static String language; 
	protected static ResourceBundle languageBundle;
	
	private static Image image = new ImageIcon("resources" + File.separator + "pea-lock.png").getImage();
	//private static Image cursorImage = new ImageIcon("resources" + File.separator + "wait_cursor.png").getImage();
	
	//private static Cursor waitCursor = Toolkit.getDefaultToolkit().createCustomCursor(cursorImage,
    //        new Point(0, 0), "wait_cursor");			
	
	private MainView (Point location) {
		
		frame = this;
		this.setTitle("Pea Factory");
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		//this.setBackground(PswDialogView.getPeaColor());
		
		language = PeaFactory.getLanguage();
		languageBundle = PeaFactory.getLanguagesBundle();
				
		DataType dataType = DataType.getCurrentType();

		//Image icon = new ImageIcon("resources" + File.separator + "pea-lock.png").getImage();
	    this.setIconImage(image);
		// will be used in Action Listener
		//waitCursor = Toolkit.getDefaultToolkit().createCustomCursor(cursorImage,
	    //        new Point((int)this.getLocation().getX(),(int) this.getLocation().getY()), "wait_cursor");		



		JPanel contentPane = (JPanel)this.getContentPane();
		contentPane.setBorder(PeaBorderFactory.createBorder(true));//(new EmptyBorder(5, 15, 5, 15));
		// use frame to collect random values: 
		contentPane.addMouseMotionListener(new MouseRandomCollector() );
		JScrollPane frameScrollPane = new JScrollPane(contentPane);
		this.setContentPane(frameScrollPane);
		
		BoxLayout boxLayout = new BoxLayout(contentPane, BoxLayout.Y_AXIS);
		contentPane.setLayout(boxLayout);
		
		JPanel menuPanel = new JPanel();
		menuPanel.setLayout(new BoxLayout(menuPanel, BoxLayout.X_AXIS));		
		// Menu:
		Menu menu = new Menu(languageBundle);
		JLabel languageLabel = new JLabel(language + "  ");
		LanguageMenu languageMenu = new LanguageMenu();
		languageMenu.setToolTipText(languageBundle.getString("tooltip_language"));

		menuPanel.add(menu);		
		menuPanel.add(Box.createHorizontalGlue() );
		menuPanel.add(languageLabel);
		menuPanel.add(languageMenu);

		contentPane.add(menuPanel);
		
		JPanel projectLabelPanel = new JPanel();
		projectLabelPanel.setLayout(new BoxLayout(projectLabelPanel, BoxLayout.X_AXIS));
		projectLabel = new JLabel();
		// show the type of the PEA: notebook, image...
		projectLabel.setText(dataType.getDescription());

		// show property of the PEA:
		if (PeaFactory.isKeyFileProperty() == false) {
			if (PeaFactory.isBlankPea() == true){
				updateLabelText(languageBundle.getString("blank_pea"));
			} else {
				updateLabelText(languageBundle.getString("default_pea"));
			}
		} else {
			if (PeaFactory.isUseKeptRandomBytes() == true) {
				updateLabelText(languageBundle.getString("only_compatible_with_previous_pea"));
				//projectLabel.setText(projectLabel.getText() + " - compatible with previouse PEA");
			} else {
				updateLabelText(languageBundle.getString("key_file_pea"));
			}
		}
		projectLabel.setFont(new Font(Font.SANS_SERIF, Font.BOLD, 14));
		projectLabel.setBorder(new LineBorder(Color.GRAY, 1));
		projectLabelPanel.add(projectLabel);
		projectLabelPanel.add(Box.createHorizontalGlue());
		contentPane.add(Box.createVerticalStrut(10));
		contentPane.add(projectLabelPanel);
		contentPane.add(Box.createVerticalStrut(10));
		
		if (PeaFactory.isBlankPea() == false) {		
			
			if (dataType instanceof NotesType || // text
					dataType instanceof EditorType ){	
				
				TextTypePanel textTypePanel = TextTypePanel.getInstance(languageBundle);
				contentPane.add(textTypePanel);			

			} else if (dataType instanceof ImageType) {	
				
				ImageTypePanel imageTypePanel = ImageTypePanel.getInstance(languageBundle);
				contentPane.add(imageTypePanel);
			
			} else if (dataType instanceof FileType) {	
				contentPane.add(Box.createVerticalStrut(20));
				if (filePanel == null) {
					filePanel = (FileTypePanel) PeaProperties.getTypePanel();
				}
				contentPane.add(filePanel);				
			}
			
			JPanel pathPanel = new JPanel();
			pathPanel.setLayout(new FlowLayout(FlowLayout.LEADING));
			pathLabel = new JLabel();

			if (openedFileName != null) {
				pathLabel.setText(openedFileName);
				pathLabel.setBorder(new LineBorder(Color.BLACK,1));
				pathPanel.add(pathLabel);
			}
			contentPane.add(pathPanel);

			
			JPanel pswPanel = new JPanel();		
			pswPanel.setLayout(new BoxLayout(pswPanel, BoxLayout.Y_AXIS));

			JPanel pswLabelPanel = new JPanel();
			pswLabelPanel.setLayout(new BoxLayout(pswLabelPanel, BoxLayout.X_AXIS));
			JLabel pswLabel = new JLabel(languageBundle.getString("psw_label"));
			pswLabelPanel.add(pswLabel);
			
			pswLabelPanel.add(Box.createHorizontalStrut(20));		
			
			qualityLabel = new JLabel();
			qualityLabel.setFont(new Font(Font.MONOSPACED, Font.BOLD, 14));			
			pswLabelPanel.add(qualityLabel);
			
			pswLabelPanel.add(Box.createHorizontalStrut(10));
			
			qualityLabelStars = new JLabel();
			qualityLabelStars.setMaximumSize(new Dimension (300, 10));
			qualityLabelStars.setOpaque(true);
			//qualityLabelStars.setBackground(Color.WHITE);
			qualityLabelStars.setFont(new Font(Font.SERIF, Font.ITALIC, 18));
			pswLabelPanel.add(qualityLabelStars);
			
			pswLabelPanel.add(Box.createHorizontalGlue());
			
			JButton charTableButton = new JButton(languageBundle.getString("char_table"));
			charTableButton.addActionListener(this);
			charTableButton.setActionCommand("charTable1");
			pswLabelPanel.add(charTableButton);
			pswPanel.add(pswLabelPanel);

			pswField = new JPasswordField(30);
			pswField.setToolTipText(languageBundle.getString("tooltip_psw"));
			// collect random values from key events:
			pswField.addKeyListener(new KeyRandomCollector() );
/*			// does not register events from CharTable!
  			pswField.addKeyListener(new KeyListener() {

				@Override
				public void keyPressed(KeyEvent arg0) {}			

				@Override
				public void keyReleased(KeyEvent arg0) {
					//System.out.println(pswField.getPassword());
					updatePasswordCheck();	
				}
				@Override
				public void keyTyped(KeyEvent arg0) {}			
			});*/
			pswField.getDocument().addDocumentListener(new DocumentListener() {

				@Override
				public void changedUpdate(DocumentEvent e) {
					//System.out.println("changed");
				}
				// if the event comes from CharTable or virtual Keyboard
				// two events are received because CharTable must replace
				// the current password (remove and insert)
				@Override
				public void insertUpdate(DocumentEvent e) {
					//System.out.println("insert");
					updatePasswordCheck();	
				}

				@Override
				public void removeUpdate(DocumentEvent e) {
					//System.out.println("removed");
					updatePasswordCheck();	
				}		
			});
			pswPanel.add(pswField);
			
			pswPanel.add(Box.createVerticalStrut(10));
			
			JPanel pswLabelPanel2 = new JPanel();
			pswLabelPanel2.setLayout(new BoxLayout(pswLabelPanel2, BoxLayout.X_AXIS));
			JLabel pswLabel2 = new JLabel(languageBundle.getString("psw_label_2"));
			pswLabel2.setPreferredSize(new Dimension(300,30));
			pswLabelPanel2.add(pswLabel2);
			pswLabelPanel2.add(Box.createHorizontalGlue());
			JButton charTableButton2 = new JButton(languageBundle.getString("char_table"));
			charTableButton2.addActionListener(this);
			charTableButton2.setActionCommand("charTable2");
			pswLabelPanel2.add(charTableButton2);
			pswPanel.add(pswLabelPanel2);


			pswField2 = new JPasswordField(30);
			pswField2.setToolTipText(languageBundle.getString("tooltip_psw2"));
			// collect random values from key events:
			pswField2.addKeyListener(new KeyRandomCollector() );
			pswPanel.add(pswField2);
			
			contentPane.add(Box.createVerticalStrut(10));
			
			contentPane.add(pswPanel);
		
		} // end if blankPea == false

		
		JPanel jarNamePanel = new JPanel();		
		jarNamePanel.setLayout(new BoxLayout(jarNamePanel, BoxLayout.Y_AXIS));
		
		JLabel jarNameLabel = new JLabel(languageBundle.getString("jar_name_label"));
		jarNameLabel.setPreferredSize(new Dimension(300,30));
		jarNamePanel.add(jarNameLabel);
		
		jarNameField = new JTextField(30);
		jarNameField.setToolTipText(languageBundle.getString("tooltip_jarname"));
		// collect random values from key events:
		jarNameField.addKeyListener(new KeyRandomCollector() );

		jarNamePanel.add(jarNameField);
		contentPane.add(Box.createVerticalStrut(10));
		contentPane.add(jarNamePanel);
	
		
		JPanel labelTextPanel = new JPanel();		
		labelTextPanel.setLayout(new BoxLayout(labelTextPanel, BoxLayout.Y_AXIS));		
		JLabel abovePswfieldLabel = new JLabel(languageBundle.getString("above_pswfield_label"));
		abovePswfieldLabel.setPreferredSize(new Dimension(300,30));
		labelTextPanel.add(abovePswfieldLabel);		
		labelTextField = new JTextField(30);
		labelTextField.setToolTipText(languageBundle.getString("tooltip_text_above"));
		// collect random values from key events:
		labelTextField.addKeyListener(new KeyRandomCollector() );
		labelTextPanel.add(labelTextField);
		contentPane.add(Box.createVerticalStrut(10));
		contentPane.add(labelTextPanel);

		
		JPanel newProjectPanel = new JPanel(new FlowLayout(FlowLayout.LEADING));
		JButton newProjectButton = new JButton(languageBundle.getString("new_project"));
		newProjectButton.setActionCommand("newProject");
		newProjectButton.addActionListener(this);
		newProjectPanel.add(newProjectButton);
		contentPane.add(Box.createVerticalStrut(10));
		contentPane.add(newProjectPanel);

		
		JPanel createFilePanel = new JPanel();
		createFilePanel.setLayout(new BoxLayout(createFilePanel, BoxLayout.X_AXIS));
		
		progressBar = new JProgressBar(0,100);
		progressBar.setValue(0);
		progressBar.setStringPainted(false);
		progressBar.setPreferredSize(new Dimension(200, 30));
		progressBar.setBorderPainted(false);// not visible if no task
		createFilePanel.add(progressBar);
		createFilePanel.add(Box.createHorizontalStrut(50));		

		
		JButton createFileButton = new JButton(languageBundle.getString("create_jarfile_button"));
		createFileButton.setActionCommand("create");
		createFileButton.addActionListener(this);
		createFileButton.setToolTipText(languageBundle.getString("tooltip_create"));
		createFilePanel.add(createFileButton);
		contentPane.add(Box.createVerticalStrut(10));
		contentPane.add(createFilePanel);

		setLocation(location);		
	
		int prefHeight = (int) this.getLayout().preferredLayoutSize(contentPane).getHeight();
		int prefWidth = (int) this.getLayout().preferredLayoutSize(contentPane).getWidth();

		// size of frame depends on screen size: 
		if ( (prefHeight + 120) > screenHeight) { // width + 50, because of scrollpanes' width
			prefHeight = (int) screenHeight - 130;
		}
		if ( (prefWidth + 130) > screenWidth) {
			prefWidth = (int) screenWidth - 150;
		}
		// set JScrollPane size for frame:
		frameScrollPane.setPreferredSize(new Dimension(prefWidth, prefHeight + 50));
	}
	/**
	 * Get the current static instance or create a new one. 
	 * 
	 * @return	the current or new frame
	 */
	public final static MainView getInstance(Point _location) {
		if (frame == null) {
			frame = new MainView(_location);
			return frame;
		} else {
			//System.err.println("MainView: existing instance.");
			return frame;
		}
	}



	@Override
	public void actionPerformed(ActionEvent ape) {
		String com = ape.getActionCommand();
		
		if (com.equals("newProject")) {
			
			Point location = new Point(100, 10);
			if (frame != null) {
				location = frame.getLocationOnScreen();
			}
			
			ProjectSelection proj = new ProjectSelection(location);
			//proj.setLocation( (int)this.getLocation().getX() + 100, (int)this.getLocation().getY() + 60 );
			proj.setVisible(true); 

			
		} else if ( com.startsWith("charTable")) {	
			if(com.equals("charTable1")){
				CharTable table = new CharTable(this, pswField, 0);
				table.setVisible(true);
			} else {
				CharTable table = new CharTable(this, pswField2, 0);
				table.setVisible(true);				
			}				
		} else if ( com.equals("create")) {	
		
			EntropyPool.getInstance().stopCollection();

			if (jarNameField.getText() == null) {
				JarStuff.setJarFileName("default.jar");				
			} else if (jarNameField.getText().length() > 0) {
				String jarText = jarNameField.getText();
				//jarText = jarText.replaceAll("[^a-zA-Z0-9.-]", "_");
				jarText = jarText.replaceAll("[:\\\\/*\"?|<>' $%!#&{}@´`+=();]", "_");
				JarStuff.setJarFileName(jarText + ".jar");
			} else {
				JarStuff.setJarFileName("default.jar");
			}
			//System.out.println("MainView - actionPerformed, jarTextField: "+ jarNameField.getText());			
			//System.out.println("MainView - actionPerformed, jarFileName: "+ JarStuff.getJarFileName());
			
			String labelText = labelTextField.getText();
			labelText = deleteDangerousChars(labelText);
			
			JarStuff.setLabelText(labelText );
			
			if (DataType.getCurrentType() instanceof ImageType
					&& ImageType.getImageName() == null && PeaFactory.isBlankPea() == false) {
				JOptionPane.showMessageDialog(frame,
						languageBundle.getString("open_image_button"),//"No image selected.",
					    "Error",
					    JOptionPane.ERROR_MESSAGE);
				return;
			}
			
			if (DataType.getCurrentType() instanceof FileType) {
				
				if (filePanel.getValidFileNames(true) != null
						&& filePanel.getValidFileNames(true).length == 1) { // only one valid file selected
					
					// ask for default file: -> openedFileName
					int n = JOptionPane.showConfirmDialog(this,
							languageBundle.getString("ask_default_file"),			    			
			    			"Warning", 
			    			JOptionPane.YES_NO_CANCEL_OPTION,
			    			JOptionPane.QUESTION_MESSAGE);
					if (n == JOptionPane.YES_OPTION) { // ok			
						MainView.setOpenedFileName(filePanel.getValidFileNames(true)[0]);
					} else if (n == JOptionPane.NO_OPTION) {
						MainView.setOpenedFileName(null);
					} else { // cancel, close
						return;
					}
				} else if (filePanel.getValidFileNames(true) == null || 
						filePanel.getValidFileNames(true).length == 0){
					
					if (PeaFactory.isKeyFileProperty() == true) {
						JOptionPane.showMessageDialog(frame,
								"No file selected.",
							    "Error",
							    JOptionPane.ERROR_MESSAGE);
						return;
					}
				} else{
					MainView.setOpenedFileName(null);
				}
			}			
			
			progressBar.setBorderPainted(true);
			progressBar.setStringPainted(true);

			PeaFactory.createFile();
					
			Point location = new Point(100, 10);
			if (frame != null) {
				location = frame.getLocationOnScreen();
				frame.dispose(); 
				frame = null;
			}				

			ProjectSelection proj = new ProjectSelection(location);
			//proj.setLocation( (int)this.getLocation().getX() + 100, (int)this.getLocation().getY() + 60 );
			proj.setVisible(true); 			
		}	
	}
	
	private String deleteDangerousChars(String text) {
		
		// these characters would allow to run code..!
		text = text.replace('"', ' ');
		text = text.replace('\'', ' ');		
		text = text.replace(';', ',');

		// these characters would cause problems for file names
		text = text.replace('\\', '-'); // windows
		text = text.replace(" ", "-"); // unix 
		text = text.replace('/', '-'); // windows and unix
		text = text.replace('!', '-'); // unix: bash problems
		
		
		// replace escape sequences: these characters will cause compiling problems
		text = text.replace('\t',  ' ');
		text = text.replace('\b', ' ');		
		text = text.replace('\n',  ' ');
		text = text.replace('\r', ' ');
		text = text.replace('\f',  ' ');
		
		return text;
	}
	/**
	 * Check the quality of the currently typed password
	 * in the first password field. Set a number of stars
	 * and the foreground of the stars (red for weak, yellow 
	 * for medium, green for strong) and a String (weak, medium, strong)
	 */
	private final void updatePasswordCheck() {
		char[] pwd = pswField.getPassword();

		int quality = PasswordQualityCheck.checkQuality(pwd);//pswField.getPassword() );
		// display at most 48 stars:
		if (quality > 48){
			quality = 48;
		}
		
		if (pwd != null){
			Zeroizer.zero(pwd);
		}
		pwd = null;
		char[] q = new char[quality];
		//Arrays.fill(q,  '*');
		Arrays.fill(q,  ' ');

		if (quality < 12) {
			qualityLabel.setText(PeaProperties.getBundle().getString("weak"));
			qualityLabelStars.setText(new String(q)); 
			//qualityLabelStars.setForeground(Color.RED);
			qualityLabelStars.setBackground(Color.RED);
		} else if (quality < 16) {
			qualityLabel.setText(PeaProperties.getBundle().getString("medium"));
			qualityLabelStars.setText(new String(q)); 
			//qualityLabelStars.setForeground(Color.YELLOW);
			qualityLabelStars.setBackground(Color.YELLOW);
		} else {
			qualityLabel.setText(PeaProperties.getBundle().getString("strong"));
			qualityLabelStars.setText(new String(q)); 
			//qualityLabelStars.setForeground(Color.GREEN);
			qualityLabelStars.setBackground(Color.GREEN);
		}
		
		//qualityLabel.repaint();
	}
	/**
	 * Dispose the existing frame and create a new one
	 * depending on the new settings. 
	 */
	public final static void updateFrame(Point _location) {
		
		if (frame != null) {
			//_location = frame.getLocationOnScreen();
			frame.dispose(); 
			frame = null;
		}			
		frame = MainView.getInstance(_location);
		frame.pack();
		frame.setVisible(true);
	}
	
	/**
	 * The label describing the PEA containing a general PEA description 
	 * is appended with a more detailed description
	 * previous detailed description is overwritten
	 *  
	 * @param newText
	 */
	public final void updateLabelText(String newText) {
		String labelText =  projectLabel.getText();
		if (labelText.contains(" - ") == true) {
			String firstPart = projectLabel.getText().substring(0, projectLabel.getText().indexOf(" - "));
			projectLabel.setText(firstPart + " - " + newText);
		} else {
			projectLabel.setText(labelText + " - " + newText);
		}
	}
	
	//==========================================
	// Getter & Setter

	/**
	 * Get the password from the first field
	 * as an array of chars
	 * 
	 * @return	the password as char array
	 */
	public static char[] getPsw() {
		return pswField.getPassword();
	}
	/**
	 * Get the password of the second password field
	 * as an array of chars
	 * 
	 * @return	the password from the second 
	 * 			password field as char array
	 */
	public static char[] getPsw2() {
		return pswField2.getPassword();
	}
	// Re-"Setter":
	/**
	 * Clear the password field
	 */
	public static void resetPsw() {
		pswField.setText("");
	}
	/**
	 * Clear the second password field
	 */
	public static void resetPsw2() {
		pswField2.setText("");
	}
	/**
	 * Get the file name to open with the produced PEA.
	 * 
	 * @return	the file name as String
	 */
	public final static String getOpenedFileName() {
		return openedFileName;
	}
	/**
	 * Set the file name to open with the produced PEA. The file
	 * name is part of the setting (compiling) of the PEA.
	 * 
	 * @param _openedFileName
	 */
	public final static void setOpenedFileName(String _openedFileName) {
		openedFileName = _openedFileName;
	}
	/**
	 * Get the current location of the frame
	 * 
	 * @return	the location as point
	 */
	public final static Point getFrameLocation() {
		return frame.getLocation();
	}
	public final static FileTypePanel getFilePanel() {
		return filePanel;
	}
	public final static void setFilePanel(FileTypePanel newFilePanel) {
		filePanel = newFilePanel;
	}
	/**
	 * Checks if the instance of MainView is actually displayed or not
	 * 
	 * @return	true if displayed
	 */
	public final static boolean isDisplayed() {// used in ProjectSelection
		if (frame == null) {
			return false; 
		} else if ( frame.isVisible() == false ) {
			return false;
		} else {
			return true;
		}
	}
	/**
	 * Get the PEA image to use as frame icon. 
	 * 
	 * @return	the PEA image
	 */
	public final static Image getImage() {
		return image;
	}

	/**
	 * Display the wait cursor to indicate that
	 * program is working
	 */
	public static void showWaitCursor() {
		frame.setCursor(CursorManager.getWaitCursor());
	}
	/**
	 * Reset the cursor to the default cursor
	 */
	public static void resetCursor() {
		// reset the cursor:
		frame.setCursor(CursorManager.getDefaultCursor());

	}
}
